/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.definition;

import java.util.List;

import org.ow2.novabpm.util.Misc;



/**
 * @author Guillaume Porcher
 *
 */
public class ActivityInstantiatorDescriptor {

  protected String variableName;
  
  protected List<Object> variableValues;
  protected int joinNumber;
  
  protected ActivityInstantiatorDescriptor() { }

  public ActivityInstantiatorDescriptor(int joinNumber, String variableName, List<Object> variableValues) {
    Misc.badStateIfTrue(joinNumber <= 0, "The join number must be greater than 0");
    Misc.checkArgsNotNull(variableName, variableValues);
    Misc.badStateIfTrue(joinNumber > variableValues.size(), "The join number must be lesser than the number of variable values");
    Misc.badStateIfTrue(variableName.length() == 0, "The variable name must not be empty");
    this.joinNumber = joinNumber;
    this.variableName = variableName;
    this.variableValues = variableValues;
  }

  
  public String getVariableName() {
    return variableName;
  }

  
  public List<Object> getVariableValues() {
    return variableValues;
  }

  
  public int getJoinNumber() {
    return joinNumber;
  }

}
