/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.definition.activity;

import java.util.Date;

import org.jbpm.pvm.Execution;
import org.ow2.bonita.definition.JavaHook;
import org.ow2.bonita.definition.Performer;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.runtime.impl.AutomaticBodyImpl;
import org.ow2.bonita.facade.runtime.impl.TaskFullInstanceImpl;
import org.ow2.bonita.facade.uuid.AutomaticBodyUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.runtime.Repository;
import org.ow2.bonita.runtime.TaskRunTime;
import org.ow2.bonita.runtime.XpdlExecution;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.EnvTool;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class NoImplementation extends AbstractActivity {

  private TaskRunTime taskRuntime = null;

  protected  NoImplementation() {
    super();
  }

  public NoImplementation(String activityName, JoinType joinType,
      SplitType splitType,
      Performer performer,
      boolean isTask) {
    super(activityName, joinType,
        splitType,
        isTask ? ActivityType.task : ActivityType.no,
            performer);
  }

  @Override
  protected boolean executeBusinessLogic(Execution execution) {
    XpdlExecution xpdlExecution = (XpdlExecution) execution;
    if (activityType.equals(ActivityType.task)) {
      addTaskToRepository(taskRuntime);
      return false;
    }
    //noImpl must have a hook if it is not a task
    HookExecutor.executeHooks(javaHooks, xpdlExecution, activityId, JavaHook.Type.automaticOnEnter);
    return true;
  }

  @Override
  protected boolean bodyStartAutomatically() {
    if (activityType.equals(ActivityType.task)) {
      return false;
    }
    return true;
  }

  @Override
  public ActivityBody getBody(XpdlExecution xpdlExecution, ActivityInstance<ActivityBody> activityInstance) {
    if (activityType.equals(ActivityType.task)) {
      return getTask(xpdlExecution, activityInstance);
    } else if (activityType.equals(ActivityType.no)) {
      return new AutomaticBodyImpl(new AutomaticBodyUUID(activityInstance.getUUID()));
    }
    throw new BonitaRuntimeException("Ouch! NoImplementation does not support type: " + activityType);
  }

  protected TaskInstance getTask(XpdlExecution xpdlExecution, ActivityInstance<ActivityBody> activityInstance) {
    this.taskRuntime = new TaskRunTime(new TaskUUID(activityInstance.getUUID()),
        xpdlExecution.getNode().getName(), this, xpdlExecution);

    final TaskUUID taskUUID = taskRuntime.getUUID();
    final TaskState state = taskRuntime.getState();
    final Date createdDate = new Date();
    final Date dueDate = taskRuntime.getDueDate();
    return new TaskFullInstanceImpl(taskUUID, state, createdDate, dueDate);
  }

  protected void addTaskToRepository(TaskRunTime taskRunTime) {
    Repository repository = EnvTool.getRepository();
    repository.addTask(taskRunTime);
    taskRunTime.ready();

  }

}


