/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.deployment;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.URL;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import org.ow2.novabpm.util.Misc;
import org.xml.sax.InputSource;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class Deployment implements Serializable {

  private static Logger log = Logger.getLogger(Deployment.class.getName());

  protected byte[] xpdl;
  protected Set<byte[]> classes = new HashSet<byte[]>();


  public Deployment(InputSource xpdl, Set<byte[]> classes) {
    Misc.checkArgsNotNull(xpdl);
    try {
      this.xpdl = Misc.getAllContentFrom(xpdl);
    } catch (IOException ioe) {
      throw new IllegalArgumentException("Couldn't open stream on input source: " + xpdl, ioe);
    }
    if (classes != null) {
      this.classes.addAll(classes);
    }
  }

  public Deployment(byte[] xpdl, Set<byte[]> classes) {
    Misc.checkArgsNotNull(xpdl);
    this.xpdl = xpdl;
    if (classes != null) {
      this.classes.addAll(classes);
    }
  }


  public Deployment(URL xpdl, Set<byte[]> classes) {
    Misc.checkArgsNotNull(xpdl);
    try {
      this.xpdl = Misc.getAllContentFrom(xpdl);
    } catch (IOException ioe) {
      throw new IllegalArgumentException("Couldn't open stream on url: " + xpdl, ioe);
    }
    if (classes != null) {
      this.classes.addAll(classes);
    }
  }


  private InputSource createInputSource(byte[] bytes) {
    InputStream inputStream = new ByteArrayInputStream(bytes);
    return new InputSource(inputStream);
  }

  public InputSource getXpdl() {
    return createInputSource(xpdl);
  }

  public Set<byte[]> getClasses() {
    return classes;
  }

}
