/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.env;

import org.ow2.bonita.persistence.db.DbHistory;
import org.ow2.novabpm.util.Misc;


/**
 * @author Guillaume Porcher
 *
 */
public class DbHistoryEnvGenerator extends EnvGenerator {

  public static final String HISTORY_HIBERNATE_CONF_NAME = "hibernate-configuration:history";
  public static final String HISTORY_HIBERNATE_SESSION_FACTORY_NAME = "hibernate-session-factory:history";
  public static final String HISTORY_HIBERNATE_SESSION_NAME = "hibernate-session:history";
  public static final String HISTORY_PERSISTENCESERVICE_NAME = "persistenceService:history";
  
  public DbHistoryEnvGenerator() {
    super();
    StringBuffer hibernateConfigXml = new StringBuffer();
    
    hibernateConfigXml
      .append("<hibernate-configuration name='" + DbHistoryEnvGenerator.HISTORY_HIBERNATE_CONF_NAME + "' >")
      .append(Misc.LINE_SEPARATOR)
      .append(EnvGenerator.defaultIndent).append("<properties resource='history.hibernate.properties' />")
      .append(Misc.LINE_SEPARATOR)
      .append(EnvGenerator.defaultIndent).append("<mappings resource='xpdl.hibernate.mappings.xml' />")
      .append(Misc.LINE_SEPARATOR)
      .append(EnvGenerator.defaultIndent).append("<cache-configuration resource='xpdl.cache.xml' usage='read-write' />")
      .append(Misc.LINE_SEPARATOR)
      .append("</hibernate-configuration>").append(Misc.LINE_SEPARATOR);
    
    setApplicationEntry(DbHistoryEnvGenerator.HISTORY_HIBERNATE_CONF_NAME, hibernateConfigXml.toString());
    
    String hibernateSessionFactory = 
      "<hibernate-session-factory "
      + "name='" + DbHistoryEnvGenerator.HISTORY_HIBERNATE_SESSION_FACTORY_NAME + "' " 
      + "configuration='" + DbHistoryEnvGenerator.HISTORY_HIBERNATE_CONF_NAME + "' />";
    setApplicationEntry(DbHistoryEnvGenerator.HISTORY_HIBERNATE_SESSION_FACTORY_NAME, hibernateSessionFactory);
    
    String hibSession = 
      "<hibernate-session name='" + DbHistoryEnvGenerator.HISTORY_HIBERNATE_SESSION_NAME + "' " 
      + "factory='" + DbHistoryEnvGenerator.HISTORY_HIBERNATE_SESSION_FACTORY_NAME + "' />";
    setBlockEntry(DbHistoryEnvGenerator.HISTORY_HIBERNATE_SESSION_NAME, hibSession);
    
    String hibPersService = 
      "<hibernate-xpdl-persistence-service " 
      + "name='" + DbHistoryEnvGenerator.HISTORY_PERSISTENCESERVICE_NAME + "' " 
      + "session='" + DbHistoryEnvGenerator.HISTORY_HIBERNATE_SESSION_NAME + "' />";
    setBlockEntry(DbHistoryEnvGenerator.HISTORY_PERSISTENCESERVICE_NAME, hibPersService);
    
    String historyArgs = "<arg><string value='" + DbHistoryEnvGenerator.HISTORY_PERSISTENCESERVICE_NAME + "' /></arg>";
    setHistoryType(DbHistory.class, historyArgs);
  }
}
