/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.def.dataType.impl;

import org.ow2.bonita.facade.def.dataType.DataTypeDefinition;
import org.ow2.bonita.facade.def.dataType.DataTypeValue;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.novabpm.util.Misc;


public class DataTypeDefinitionImpl implements DataTypeDefinition {

  protected Type type;
  protected DataTypeValue value;

  protected DataTypeDefinitionImpl() { }

  public DataTypeDefinitionImpl(Type type, DataTypeValue value) {
    super();
    this.type = type;
    this.value = value;
    if (type == null) {
      throw new BonitaRuntimeException("DataType type cannot be null.");
    }
    if (value == null) {
      throw new BonitaRuntimeException("Null dataType value");
    }
    boolean supported = false;
    supported = supported || value instanceof BasicTypeDefinitionImpl;
    supported = supported || value instanceof EnumerationTypeDefinitionImpl;
    if (!supported) {
      throw new BonitaRuntimeException("Unsupported DataType : " + value);
    }
  }

  public DataTypeDefinitionImpl(DataTypeDefinition src) {
    Misc.badStateIfNull(src, "Ouch! Impossible to build a new Definition object from null");
    this.type = src.getType();
    DataTypeValue value = src.getValue();
    if (value != null) {
      this.value = value.copy();
    }
  }

  public Type getType() {
    return type;
  }

  public DataTypeValue getValue() {
    return value;
  }

}


