/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.def.majorElement;

import java.io.Serializable;
import java.util.Set;

import org.ow2.bonita.facade.uuid.ActivitySetDefinitionUUID;

/**
 * This interface represents the ActivitySet definition.
 * It's derived from the ActivitySet of XPDL.
 *
 */
public interface ActivitySetDefinition extends Serializable, ProcessDefinitionRecord {
  /**
   * Returns the UUID for the ActivitySetDefinition.
   */
  ActivitySetDefinitionUUID getUUID();
  /**
   * Returns the value of the id attribute of the ActivitySet element defined into the XPDL file.
   */
  String getActivitySetId();
  /**
   * An activitySet is a map of activities and transitions.<br>
   * An activity may be a block activity that executes an activity set.
   * @return the set of {@link ActivityDefinition} interfaces.
   */
  Set<ActivityDefinition> getActivities();
  /**
   * Returns the set of transitions defined within the block activity.
   */
  Set<TransitionDefinition> getTransitions();
}


