/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.def.majorElement.impl;

import org.ow2.bonita.facade.def.dataType.DataTypeDefinition;
import org.ow2.bonita.facade.def.dataType.impl.DataTypeDefinitionImpl;
import org.ow2.bonita.facade.def.majorElement.DataFieldDefinition;
import org.ow2.bonita.facade.uuid.DataFieldDefinitionUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;


public class DataFieldDefinitionImpl extends ProcessDefinitionRecordImpl implements DataFieldDefinition {

  protected DataFieldDefinitionUUID uuid;
  protected String dataFieldId;
  protected DataTypeDefinition dataType;
  protected String description;
  protected String initialValue;
  protected String length;
  protected boolean array;
  protected boolean activityDataField;

  protected DataFieldDefinitionImpl() { }

  public DataFieldDefinitionImpl(DataFieldDefinitionUUID uuid, String dataFieldId,
      PackageDefinitionUUID packageUUID, ProcessDefinitionUUID processUUID,
      DataTypeDefinition dataType, String description, String initialValue, String length, String name, boolean array,
      boolean activityDataField) {
    super(packageUUID, name, processUUID);
    this.uuid = uuid;
    this.dataFieldId = dataFieldId;
    this.dataType = dataType;
    this.description = description;
    this.initialValue = initialValue;
    this.length = length;
    this.array = array;
    this.activityDataField = activityDataField;
  }

  public DataFieldDefinitionImpl(DataFieldDefinition src) {
    super(src);
    this.uuid = new DataFieldDefinitionUUID(src.getUUID());
    this.dataFieldId = src.getDataFieldId();
    if (src.getDataType() != null) {
      this.dataType = new DataTypeDefinitionImpl(src.getDataType());
    }
    this.description = src.getDescription();
    this.initialValue = src.getInitialValue();
    this.length = src.getLength();
    this.array = src.isArray();
    this.activityDataField = src.isActivityDataField();
  }

  public DataTypeDefinition getDataType() {
    return dataType;
  }

  public String getDescription() {
    return description;
  }

  public String getInitialValue() {
    return initialValue;
  }

  public String getLength() {
    return length;
  }

  public boolean isArray() {
    return array;
  }

  public boolean isActivityDataField() {
    return activityDataField;
  }

  public String getDataFieldId() {
    return dataFieldId;
  }

  public DataFieldDefinitionUUID getUUID() {
    return uuid;
  }
}


