/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.def.majorElement.impl;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import org.ow2.bonita.facade.def.element.ExternalPackageDefinition;
import org.ow2.bonita.facade.def.element.TypeDeclarationDefinition;
import org.ow2.bonita.facade.def.element.impl.ExternalPackageDefinitionImpl;
import org.ow2.bonita.facade.def.element.impl.TypeDeclarationDefinitionImpl;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.util.Tool;


public class PackageDefinitionImpl extends DefinitionRecordImpl implements PackageDefinition {

  protected PackageDefinitionUUID uuid;
  protected String packageId;
  protected Date deployedDate;
  protected Date undeployedDate;
  protected String deployedBy;
  protected String undeployedBy;

  protected String author;
  protected Set<String> classDependencies;
  protected String codepage;
  protected GraphConformance conformanceClass;
  protected String costUnit;
  protected String countrykey;
  protected String created;
  protected String description;
  protected String documentation;
  protected Set<ExternalPackageDefinition> externalPackages;
  protected Set<String> packageDependencies;
  protected String priorityUnit;
  protected PublicationStatus publicationStatus;
  protected Set<String> responsibles;
  protected String scriptType;
  protected String scriptVersion;
  protected String scriptGrammar;
  protected Set<TypeDeclarationDefinition> typeDeclarations;
  protected String vendor;
  protected String version;
  protected String xpdlVersion;

  protected PackageDefinitionImpl() { }

  PackageDefinitionImpl(PackageDefinitionUUID uuid) {
    super(uuid, null);
  }

  protected PackageDefinitionImpl(PackageDefinitionUUID uuid, Date deployedDate, String deployedBy,
      Date undeployedDate, String undeployedBy,
      String packageId, String name, String author, Set<String> classDependencies,
      String codepage, GraphConformance conformanceClass, String costUnit,
      String countrykey, String created, String description, String documentation,
      Set<ExternalPackageDefinition> externalPackages, Set<String> packageDependencies,
      String priorityUnit, PublicationStatus publicationStatus,
      Set<String> responsibles, String scriptType, String scriptVersion, String scriptGrammar,
      Set<TypeDeclarationDefinition> typeDeclarations, String vendor, String version,
      String xpdlVersion) {
    super(uuid, name);
    this.uuid = uuid;
    this.packageId = packageId;
    this.deployedDate = deployedDate;
    this.deployedBy = deployedBy;
    this.undeployedDate = undeployedDate;
    this.undeployedBy = undeployedBy;

    this.author = author;
    this.classDependencies = classDependencies;
    this.codepage = codepage;
    this.conformanceClass = conformanceClass;
    this.costUnit = costUnit;
    this.countrykey = countrykey;
    this.created = created;
    this.description = description;
    this.documentation = documentation;
    this.externalPackages = externalPackages;
    this.packageDependencies = packageDependencies;
    this.priorityUnit = priorityUnit;
    this.publicationStatus = publicationStatus;
    this.responsibles = responsibles;
    this.scriptType = scriptType;
    this.scriptVersion = scriptVersion;
    this.scriptGrammar = scriptGrammar;
    this.typeDeclarations = typeDeclarations;
    this.vendor = vendor;
    this.version = version;
    this.xpdlVersion = xpdlVersion;
  }

  public PackageDefinitionImpl(PackageDefinition src) {
    super(src);
    this.uuid = new PackageDefinitionUUID(src.getUUID());
    this.packageId = src.getPackageId();
    this.deployedDate = Tool.copy(src.getDeployedDate());
    this.undeployedDate = Tool.copy(src.getUndeployedDate());
    this.deployedBy = src.getDeployedBy();
    this.undeployedBy = src.getUndeployedBy();
    this.author = src.getAuthor();
    this.classDependencies = Tool.copy(src.getClassDependencies());
    this.codepage = src.getCodepage();
    this.conformanceClass = src.getConformanceClass();
    this.costUnit = src.getCostUnit();
    this.countrykey = src.getCountrykey();
    this.created = src.getCreated();
    this.description = src.getDescription();
    this.documentation = src.getDocumentation();
    Set<ExternalPackageDefinition> externalPackages = src.getExternalPackages();
    if (externalPackages != null) {
      this.externalPackages = new HashSet<ExternalPackageDefinition>();
      for (ExternalPackageDefinition d : externalPackages) {
        this.externalPackages.add(new ExternalPackageDefinitionImpl(d));
      }
    }
    this.packageDependencies = Tool.copy(src.getPackageDependencies());
    this.priorityUnit = src.getPriorityUnit();
    this.publicationStatus = src.getPublicationStatus();
    this.responsibles = Tool.copy(src.getResponsibles());
    this.scriptType = src.getScriptType();
    this.scriptVersion = src.getScriptVersion();
    this.scriptGrammar = src.getScriptGrammar();
    Set<TypeDeclarationDefinition> typeDeclarations = src.getTypeDeclarations();
    if (typeDeclarations != null) {
      this.typeDeclarations = new HashSet<TypeDeclarationDefinition>();
      for (TypeDeclarationDefinition d : typeDeclarations) {
        this.typeDeclarations.add(new TypeDeclarationDefinitionImpl(d));
      }
    }
    this.vendor = src.getVendor();
    this.version = src.getVersion();
    this.xpdlVersion = src.getXPDLVersion();
  }

  public String toString() {
    return this.getClass().getName()
      + "[uuid: " + getUUID()
      + ", packageDefinitionUUID: " + getPackageDefinitionUUID()
      + ", packageId: " + getPackageId() + "]";
  }

  public String getAuthor() {
    return author;
  }

  public String getCodepage() {
    return codepage;
  }

  public GraphConformance getConformanceClass() {
    return conformanceClass;
  }

  public String getCostUnit() {
    return costUnit;
  }

  public String getCountrykey() {
    return countrykey;
  }

  public String getCreated() {
    return created;
  }

  public String getDescription() {
    return description;
  }

  public String getDocumentation() {
    return documentation;
  }

  public Set<ExternalPackageDefinition> getExternalPackages() {
    return externalPackages;
  }

  public String getPriorityUnit() {
    return priorityUnit;
  }

  public PublicationStatus getPublicationStatus() {
    return publicationStatus;
  }

  public Set<String> getResponsibles() {
    return responsibles;
  }

  public String getScriptType() {
    return scriptType;
  }

  public String getScriptVersion() {
    return scriptVersion;
  }

  public String getScriptGrammar() {
    return scriptGrammar;
  }

  public Set<TypeDeclarationDefinition> getTypeDeclarations() {
    return typeDeclarations;
  }

  public String getVendor() {
    return vendor;
  }

  public String getVersion() {
    return version;
  }

  public String getXPDLVersion() {
    return xpdlVersion;
  }


  public Set<String> getClassDependencies() {
    return classDependencies;
  }

  public Set<String> getPackageDependencies() {
    return packageDependencies;
  }

  public String getXpdlVersion() {
    return xpdlVersion;
  }

  public Date getDeployedDate() {
    return deployedDate;
  }

  public String getDeployedBy() {
    return deployedBy;
  }

  public Date getUndeployedDate() {
    return undeployedDate;
  }


  public String getUndeployedBy() {
    return undeployedBy;
  }

  public String getPackageId() {
    return packageId;
  }

  public PackageDefinitionUUID getUUID() {
    return uuid;
  }
}


