/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.def.majorElement.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.ow2.bonita.facade.def.element.FormalParameterDefinition;
import org.ow2.bonita.facade.def.element.TimeEstimationDefinition;
import org.ow2.bonita.facade.def.element.impl.FormalParameterDefinitionImpl;
import org.ow2.bonita.facade.def.element.impl.TimeEstimationDefinitionImpl;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.util.Tool;


public class ProcessDefinitionImpl extends ProcessDefinitionRecordImpl implements ProcessDefinition {

  protected long dbid;

  protected ProcessDefinitionUUID uuid;
  protected String processId;
  protected AccessLevel accessLevel;
  protected String author;
  protected String codepage;
  protected String countrykey;
  protected String created;
  protected String description;
  protected DurationUnit durationUnit;
  protected List<FormalParameterDefinition> formalParameters;
  protected String limit;
  protected String priority;
  protected PackageDefinition.PublicationStatus publicationStatus;
  protected Set<String> responsibles;
  protected TimeEstimationDefinition timeEstimation;
  protected String validFrom;
  protected String validTo;
  protected String version;

  protected ProcessDefinitionImpl() { }

  protected ProcessDefinitionImpl(ProcessDefinitionUUID uuid, PackageDefinitionUUID packageUUID,
      AccessLevel accessLevel, String author, String codepage,
      String countrykey, String created, String description,
      DurationUnit durationUnit,
      List<FormalParameterDefinition> formalParameters, String processId, String limit,
      String name, String priority, PackageDefinition.PublicationStatus publicationStatus,
      Set<String> responsibles, TimeEstimationDefinition timeEstimation,
      String validFrom, String validTo, String version) {
    super(packageUUID, name, uuid);
    this.uuid = uuid;
    this.processId = processId;
    this.accessLevel = accessLevel;
    this.author = author;
    this.codepage = codepage;
    this.countrykey = countrykey;
    this.created = created;
    this.description = description;
    this.durationUnit = durationUnit;
    this.formalParameters = formalParameters;
    this.limit = limit;
    this.priority = priority;
    this.publicationStatus = publicationStatus;
    this.responsibles = responsibles;
    this.timeEstimation = timeEstimation;
    this.validFrom = validFrom;
    this.validTo = validTo;
    this.version = version;
  }

  public ProcessDefinitionImpl(ProcessDefinition src) {
    super(src);
    this.uuid = new ProcessDefinitionUUID(src.getUUID());
    this.processId = src.getProcessId();
    this.accessLevel = src.getAccessLevel();
    this.author = src.getAuthor();
    this.codepage = src.getCodepage();
    this.countrykey = src.getCountrykey();
    this.created = src.getCreated();
    this.description = src.getDescription();
    this.durationUnit = src.getDurationUnit();
    List<FormalParameterDefinition> formalParameters = src.getFormalParameters();
    if (formalParameters != null) {
      this.formalParameters = new ArrayList<FormalParameterDefinition>();
      for (FormalParameterDefinition d : formalParameters) {
        this.formalParameters.add(new FormalParameterDefinitionImpl(d));
      }
    }
    this.limit = src.getLimit();
    this.priority = src.getPriority();
    this.publicationStatus = src.getPublicationStatus();
    this.responsibles = Tool.copy(src.getResponsibles());
    if (src.getTimeEstimation() != null) {
      this.timeEstimation = new TimeEstimationDefinitionImpl(src.getTimeEstimation());
    }
    this.validFrom = src.getValidFrom();
    this.validTo = src.getValidTo();
    this.version = src.getVersion();
  }

  public String toString() {
    String st = this.getClass().getName()
      + "[uuid: " + getUUID()
      + ", packageDefinitionUUID: " + getPackageDefinitionUUID()
      + ", processDefinitionUUID: " + getProcessDefinitionUUID()
      + ", processId:" + getProcessId()
      + ", accessLevel:" + getAccessLevel()
      + ", author:" + getAuthor()
      + ", codepage:" + getCodepage()
      + ", countrykey:" + getCountrykey()
      + ", created:" + getCreated()
      + ", description:" + getDescription()
      + ", durationUnit:" + getDurationUnit()
      + ", formalParameters:" + getFormalParameters()
      + ", limit:" + getLimit()
      + ", priority:" + getPriority()
      + ", publicationStatus:" + getPublicationStatus()
      + ", responsibles:" + getResponsibles()
      + ", timeEstimation:" + getTimeEstimation()
      + ", validFrom:" + getValidFrom()
      + ", validTo:" + getValidTo()
      + ", version:" + getVersion();
    st +=  "]";
    return st;
  }

  public AccessLevel getAccessLevel() {
    return accessLevel;
  }

  public String getAuthor() {
    return author;
  }

  public String getCodepage() {
    return codepage;
  }

  public String getCountrykey() {
    return countrykey;
  }

  public String getCreated() {
    return created;
  }

  public String getDescription() {
    return description;
  }

  public DurationUnit getDurationUnit() {
    return durationUnit;
  }

  public List<FormalParameterDefinition> getFormalParameters() {
    return formalParameters;
  }

  public String getLimit() {
    return limit;
  }

  public String getPriority() {
    return priority;
  }

  public Set<String> getResponsibles() {
    return responsibles;
  }

  public TimeEstimationDefinition getTimeEstimation() {
    return timeEstimation;
  }

  public String getValidFrom() {
    return validFrom;
  }

  public String getValidTo() {
    return validTo;
  }

  public String getVersion() {
    return version;
  }

  public PackageDefinition.PublicationStatus getPublicationStatus() {
    return publicationStatus;
  }

  public String getProcessId() {
    return this.processId;
  }

  public ProcessDefinitionUUID getUUID() {
    return uuid;
  }
  
  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    ProcessDefinitionImpl other = (ProcessDefinitionImpl)obj;
    return other.getUUID().equals(uuid);
  }

  @Override
  public int hashCode() {
    return uuid.hashCode();
  }
}


