/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.def.majorElement.impl;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.bonita.facade.def.element.FormalParameterDefinition;
import org.ow2.bonita.facade.def.element.TimeEstimationDefinition;
import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.ActivitySetDefinition;
import org.ow2.bonita.facade.def.majorElement.ApplicationDefinition;
import org.ow2.bonita.facade.def.majorElement.DataFieldDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.ParticipantDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.def.majorElement.TransitionDefinition;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;


public class ProcessFullDefinitionImpl extends ProcessDefinitionImpl implements ProcessFullDefinition {

  protected Set<DataFieldDefinition> dataFields;
  protected Set<ParticipantDefinition> participants;
  protected Set<ActivityDefinition> activities;
  protected Set<TransitionDefinition> transitions;
  protected Set<ActivitySetDefinition> activitySets;
  protected Set<ApplicationDefinition> applications;

  protected ProcessFullDefinitionImpl() { }

  public ProcessFullDefinitionImpl(ProcessDefinitionUUID uuid,
      PackageDefinitionUUID packageUUID, AccessLevel accessLevel, String author,
      String codepage, String countrykey, String created, String description,
      DurationUnit durationUnit,
      List<FormalParameterDefinition> formalParameters, String id, String limit,
      String name, String priority, PackageDefinition.PublicationStatus publicationStatus,
      Set<String> responsibles, TimeEstimationDefinition timeEstimation,
      String validFrom, String validTo, String version,
      Set<DataFieldDefinition> dataFields, Set<ParticipantDefinition> participants,
      Set<ActivityDefinition> activities, Set<TransitionDefinition> transitions,
      Set<ActivitySetDefinition> activitySets, Set<ApplicationDefinition> applications) {
    super(uuid, packageUUID, accessLevel, author, codepage, countrykey, created, description,
        durationUnit, formalParameters, id, limit, name,
        priority, publicationStatus, responsibles, timeEstimation, validFrom, validTo, version);
    this.dataFields = dataFields;
    this.participants = participants;
    this.activities = activities;
    this.transitions = transitions;
    this.activitySets = activitySets;
    this.applications = applications;
  }

  public ProcessFullDefinitionImpl(ProcessFullDefinition src) {
    super(src);
    this.processId = src.getProcessId();
    Set<DataFieldDefinition> dataFields = src.getDataFields();
    if (dataFields != null) {
      this.dataFields = new HashSet<DataFieldDefinition>();
      for (DataFieldDefinition d : dataFields) {
        this.dataFields.add(new DataFieldDefinitionImpl(d));
      }
    }
    Set<ParticipantDefinition> participants = src.getParticipants();
    if (participants != null) {
      this.participants = new HashSet<ParticipantDefinition>();
      for (ParticipantDefinition d : participants) {
        this.participants.add(new ParticipantDefinitionImpl(d));
      }
    }
    Set<ActivityDefinition> activities = src.getActivities();
    if (activities != null) {
      this.activities = new HashSet<ActivityDefinition>();
      for (ActivityDefinition d : activities) {
        this.activities.add(new ActivityDefinitionImpl(d));
      }
    }
    Set<TransitionDefinition> transitions = src.getTransitions();
    if (transitions != null) {
      this.transitions = new HashSet<TransitionDefinition>();
      for (TransitionDefinition d : transitions) {
        this.transitions.add(new TransitionDefinitionImpl(d));
      }
    }
    Set<ActivitySetDefinition> activitySets = src.getActivitySets();
    if (activitySets != null) {
      this.activitySets = new HashSet<ActivitySetDefinition>();
      for (ActivitySetDefinition d : activitySets) {
        this.activitySets.add(new ActivitySetDefinitionImpl(d));
      }
    }
    Set<ApplicationDefinition> applications = src.getApplications();
    if (applications != null) {
      this.applications = new HashSet<ApplicationDefinition>();
      for (ApplicationDefinition d : applications) {
        this.applications.add(new ApplicationDefinitionImpl(d));
      }
    }
  }

  public Set<DataFieldDefinition> getDataFields() {
    return dataFields;
  }


  public Set<ParticipantDefinition> getParticipants() {
    return participants;
  }


  public Set<ActivityDefinition> getActivities() {
    return activities;
  }


  public Set<TransitionDefinition> getTransitions() {
    return transitions;
  }


  public Set<ActivitySetDefinition> getActivitySets() {
    return activitySets;
  }

  public Set<ApplicationDefinition> getApplications() {
    return applications;
  }

}


