package org.ow2.bonita.facade.ejb;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import javax.interceptor.InvocationContext;

import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.EJBUtil;
import org.ow2.bonita.util.StandaloneAccessorUtil;

/**
 * Interceptor class to redirect ejb3 bean calls to the standalone api
 * @author Thomas Gueze
 *
 */
public class EJB3Interceptor {

  private static final int SUFFIX_LENGTH = "Bean".length();

  private Object getStandaloneAPI(String className) {
    try {
      Class< ? > standaloneAccessorClass = StandaloneAccessorUtil.class;
      Method m = standaloneAccessorClass.getDeclaredMethod("getStandalone" + className);
      return m.invoke(null);
    } catch (Exception e) {
      throw new BonitaRuntimeException("Cannot get the standalone api class", e);
    }
  }

  public Object performInterception(InvocationContext ctx) throws Exception {
    String className = ctx.getMethod().getDeclaringClass().getSimpleName();
    className = className.substring(0, className.length() - SUFFIX_LENGTH);
    Object standaloneAPI = getStandaloneAPI(className);
    Method implMethod = null;
    try {
      implMethod = standaloneAPI.getClass().getMethod(ctx.getMethod().getName(), ctx.getMethod().getParameterTypes());
      return implMethod.invoke(standaloneAPI, ctx.getParameters());
    } catch (Exception e) {
      if (e instanceof InvocationTargetException && implMethod != null) {
        Throwable invocationExceptionCause = e.getCause();
        EJBUtil.manageInvokeExceptionCause(implMethod, invocationExceptionCause);
      }
      throw new BonitaRuntimeException("Cannot call the method " + ctx.getMethod().getName() + " on the standalone api", e);
    }
  }
}
