/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb;

import java.net.URL;
import java.rmi.RemoteException;
import java.util.Map;
import java.util.Set;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.jbpm.pvm.internal.env.Authentication;
import org.ow2.bonita.deployment.Deployment;
import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.DeploymentException;
import org.ow2.bonita.facade.exception.PackageNotFoundException;
import org.ow2.bonita.facade.exception.UndeletablePackageException;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.util.StandaloneAccessorUtil;

public class ManagementAPIBean implements ManagementAPI, SessionBean {

  protected SessionContext ctx = null;
  
  protected ManagementAPI managementAPI;
  
  protected ManagementAPI getAPI() {
    Authentication.setUserId(ctx.getCallerPrincipal().getName());
    if (managementAPI == null) {
      managementAPI = StandaloneAccessorUtil.getStandaloneManagementAPI();
    }
    return managementAPI;
  }

  public Map<String, ProcessDefinition> deploy(Deployment deployment) throws DeploymentException {
    return getAPI().deploy(deployment);
  }

  public Map<String, ProcessDefinition> deployBar(URL barFileURL) throws DeploymentException {
    return getAPI().deployBar(barFileURL);
  }

  public Map<String, ProcessDefinition> deployBar(byte[] barFile) throws DeploymentException {
    return getAPI().deployBar(barFile);
  }

  public void deployClass(byte[] clazz) throws DeploymentException {
    getAPI().deployClass(clazz);
  }
  public void deployClasses(Set<byte[]> classes) throws DeploymentException {
    getAPI().deployClasses(classes);
  }
  public void deployClassesInJar(byte[] classesArchive) throws DeploymentException {
    getAPI().deployClassesInJar(classesArchive);
  }
  public void undeploy(PackageDefinitionUUID packageUUID) throws DeploymentException {
    getAPI().undeploy(packageUUID);
  }
  public void removeClass(String className) throws DeploymentException {
    getAPI().removeClass(className);
  }
  public void removeClasses(String[] classNames) throws DeploymentException {
    getAPI().removeClasses(classNames);
  }
  public void replaceClass(String className, byte[] newClazz) throws DeploymentException {
    getAPI().replaceClass(className, newClazz);
  }
  public void clearHistory() {
    getAPI().clearHistory();
  }
  public Map<String, ProcessDefinition> deploy(URL xpdlURL, Class< ? >[] classes) throws DeploymentException {
    return getAPI().deploy(xpdlURL, classes);
  }
  public Map<String, ProcessDefinition> deployXpdl(URL xpdlURL) throws DeploymentException {
    return getAPI().deployXpdl(xpdlURL);
  }
  public Map<String, ProcessDefinition> deployXpdl(byte[] xpdlFile) throws DeploymentException {
    return getAPI().deployXpdl(xpdlFile);
  }
  public Map<String, ProcessDefinition> deployZip(byte[] zipFile) throws DeploymentException {
    return getAPI().deployZip(zipFile);
  }
  public Map<String, ProcessDefinition> deployZip(URL zipURL) throws DeploymentException {
    return getAPI().deployZip(zipURL);
  }
  public void deletePackage(PackageDefinitionUUID pacDefinitionUUID)
    throws PackageNotFoundException, UndeletablePackageException {
    getAPI().deletePackage(pacDefinitionUUID);
  }
  public void ejbCreate() {

  }

  public void ejbActivate() throws RemoteException {

  }

  public void ejbPassivate() throws RemoteException {

  }

  public void ejbRemove() throws RemoteException {

  }

  public void setSessionContext(SessionContext arg0) throws RemoteException {
    this.ctx = arg0;
  }
}


