/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.jbpm.pvm.internal.env.Authentication;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.AmbiguousActivityException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.StandaloneAccessorUtil;

/**
 * @author Pierre Vigneras, Charles Souillard, Guillaume Porcher
 * @date Feb 15, 2008
 */
public class QueryRuntimeAPIBean implements QueryRuntimeAPI, SessionBean {

  protected SessionContext ctx = null;

  protected QueryRuntimeAPI queryRuntimeAPI;

  protected QueryRuntimeAPI getAPI() {
    Authentication.setUserId(ctx.getCallerPrincipal().getName());
    if (queryRuntimeAPI == null) {
      queryRuntimeAPI = StandaloneAccessorUtil.getStandaloneQueryRuntimeAPI();
    }
    return queryRuntimeAPI;
  }

  public Set<ActivityInstance<ActivityBody>> getActivityInstances(
      ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    return getAPI().getActivityInstances(instanceUUID);
  }

  public Set<ActivityInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID,
      String activityId) throws InstanceNotFoundException, ActivityNotFoundException {
    return getAPI().getActivityInstances(instanceUUID, activityId);
  }

  public ActivityInstance<ActivityBody> getActivityInstance(
      ActivityInstanceUUID activityUUID) throws ActivityNotFoundException {
    return getAPI().getActivityInstance(activityUUID);
  }

  public ActivityInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID,
      String activityId) throws InstanceNotFoundException, ActivityNotFoundException, AmbiguousActivityException {
    return getAPI().getActivityInstance(instanceUUID, activityId);
  }

  public ProcessInstance getProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    return getAPI().getProcessInstance(instanceUUID);
  }

  public Set<ProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    return getAPI().getProcessInstances(processUUID);
  }

  public ActivityInstance<TaskInstance> getTask(TaskUUID taskUUID) throws TaskNotFoundException {
    return getAPI().getTask(taskUUID);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(ProcessInstanceUUID instanceUUID,
      TaskState taskState) throws InstanceNotFoundException {
    return getAPI().getTaskList(instanceUUID, taskState);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(ProcessInstanceUUID instanceUUID,
      String userId, TaskState taskState) throws InstanceNotFoundException {
    return getAPI().getTaskList(instanceUUID, taskState);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(TaskState taskState) {
    return getAPI().getTaskList(taskState);
  }

  public Collection<ActivityInstance<TaskInstance>> getTaskList(String userId, TaskState taskState) {
    return getAPI().getTaskList(taskState);
  }

  public Set<ActivityInstance<TaskInstance>> getTasks(
      ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    return getAPI().getTasks(instanceUUID);
  }

  public Object getActivityInstanceVariable(ProcessInstanceUUID instanceUUID,
      String activityId, String variableId) throws InstanceNotFoundException,
      ActivityNotFoundException, AmbiguousActivityException,
      VariableNotFoundException {
    return getAPI().getActivityInstanceVariable(instanceUUID, activityId, variableId);
  }

  public Object getActivityInstanceVariable(ActivityInstanceUUID activityUUID,
      String variableId) throws ActivityNotFoundException,
      VariableNotFoundException {
    return getAPI().getActivityInstanceVariable(activityUUID, variableId);
  }

  public Object getProcessInstanceVariable(ProcessInstanceUUID instanceUUID,
      String variableId) throws InstanceNotFoundException, VariableNotFoundException {
    return getAPI().getProcessInstanceVariable(instanceUUID, variableId);
  }

  public Map<String, Object> getActivityInstanceVariables(ProcessInstanceUUID instanceUUID,
      String activityId) throws ActivityNotFoundException,
      AmbiguousActivityException, InstanceNotFoundException {
    return getAPI().getActivityInstanceVariables(instanceUUID, activityId);
  }

  public Map<String, Object> getActivityInstanceVariables(ActivityInstanceUUID activityUUID) throws ActivityNotFoundException {
    return getAPI().getActivityInstanceVariables(activityUUID);
  }

  public Map<String, Object> getProcessInstanceVariables(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    return getAPI().getProcessInstanceVariables(instanceUUID);
  }
  public Object getVariable(ProcessInstanceUUID instanceUUID,
      String activityId, String variableId) throws InstanceNotFoundException,
      ActivityNotFoundException, VariableNotFoundException, AmbiguousActivityException {
    return getAPI().getVariable(instanceUUID, activityId, variableId);
  }
  public Map<String, Object> getVariables(ProcessInstanceUUID instanceUUID,
      String activityId) throws InstanceNotFoundException, ActivityNotFoundException, AmbiguousActivityException {
    return getAPI().getVariables(instanceUUID, activityId);
  }
  public ActivityInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId)
    throws InstanceNotFoundException, ActivityNotFoundException {
    return getAPI().getActivityInstance(instanceUUID, activityId, iterationId, activityInstanceId);
  }
  
  public Object getActivityInstanceVariable(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId,
    String variableId) throws InstanceNotFoundException, ActivityNotFoundException, VariableNotFoundException {
    return getAPI().getActivityInstanceVariable(instanceUUID, activityId, iterationId, activityInstanceId, variableId);
  }
  
  public Map<String, Object> getActivityInstanceVariables(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId)
    throws ActivityNotFoundException, InstanceNotFoundException {
    return getAPI().getActivityInstanceVariables(instanceUUID, activityId, iterationId, activityInstanceId);
  }

  public void ejbCreate() {

  }

  public void ejbActivate() throws RemoteException {

  }

  public void ejbPassivate() throws RemoteException {

  }

  public void ejbRemove() throws RemoteException {

  }

  public void setSessionContext(SessionContext arg0) throws RemoteException {
    this.ctx = arg0;
  }
}


