/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.ejb;

import java.rmi.RemoteException;
import java.util.Map;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.jbpm.pvm.internal.env.Authentication;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.AmbiguousActivityException;
import org.ow2.bonita.facade.exception.IllegalTaskStateException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.StandaloneAccessorUtil;

public class RuntimeAPIBean implements RuntimeAPI, SessionBean {

  protected SessionContext ctx = null;

  protected RuntimeAPI runtimeAPI;

  protected RuntimeAPI getAPI() {
    Authentication.setUserId(ctx.getCallerPrincipal().getName());
    if (runtimeAPI == null) {
      runtimeAPI = StandaloneAccessorUtil.getStandaloneRuntimeAPI();
    }
    return runtimeAPI;
  }

  public void startTask(TaskUUID taskUUID, boolean assignTask) throws TaskNotFoundException, IllegalTaskStateException {
    getAPI().startTask(taskUUID, assignTask);
  }

  public void finishTask(TaskUUID taskUUID, boolean taskAssign) throws TaskNotFoundException, IllegalTaskStateException {
    getAPI().finishTask(taskUUID, true);
  }

  public void resumeTask(TaskUUID taskUUID, boolean taskAssign) throws TaskNotFoundException,
  IllegalTaskStateException {
    getAPI().resumeTask(taskUUID, false);
  }

  public void suspendTask(TaskUUID taskUUID, boolean assignTask) throws TaskNotFoundException,
  IllegalTaskStateException {
    getAPI().suspendTask(taskUUID, true);
  }

  public void assignTask(TaskUUID taskUUID) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID);
  }

  public void assignTask(TaskUUID taskUUID, String actorId) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID);
  }

  public void assignTask(TaskUUID taskUUID, java.util.Set<java.lang.String> candidates) throws TaskNotFoundException {
    getAPI().assignTask(taskUUID);
  }

  public void deleteAllProcessInstances(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    getAPI().deleteAllProcessInstances(processUUID);
  }

  public void deleteProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException {
    getAPI().deleteProcessInstance(instanceUUID);
  }

  public ProcessInstanceUUID instantiateProcess(
      ProcessDefinitionUUID processUUID) throws ProcessNotFoundException {
    return getAPI().instantiateProcess(processUUID);
  }

  public ProcessInstanceUUID instantiateProcess(
      ProcessDefinitionUUID processUUID, Map<String, Object> variables) throws ProcessNotFoundException {
    return getAPI().instantiateProcess(processUUID, variables);
  }

  public void setProcessInstanceVariable(ProcessInstanceUUID instanceUUID, String variableId,
      Object variableValue) throws InstanceNotFoundException, VariableNotFoundException {
    getAPI().setProcessInstanceVariable(instanceUUID, variableId, variableValue);
  }

  public void setActivityInstanceVariable(ProcessInstanceUUID instanceUUID, String activityId, String variableId,
      Object variableValue) throws InstanceNotFoundException, ActivityNotFoundException,
      VariableNotFoundException, AmbiguousActivityException {
    getAPI().setActivityInstanceVariable(instanceUUID, activityId, variableId, variableValue);
  }

  public void setActivityInstanceVariable(ActivityInstanceUUID activityUUID,
      String variableId, Object variableValue) throws ActivityNotFoundException, VariableNotFoundException {
    getAPI().setActivityInstanceVariable(activityUUID, variableId, variableValue);
  }

  public void setVariable(ProcessInstanceUUID instanceUUID, String activityId,
      String variableId, Object variableValue) throws InstanceNotFoundException, ActivityNotFoundException,
      VariableNotFoundException, AmbiguousActivityException {
    getAPI().setVariable(instanceUUID, activityId, variableId, variableValue);
  }

  public void ejbCreate() {

  }

  public void ejbActivate() throws RemoteException {

  }

  public void ejbPassivate() throws RemoteException {

  }

  public void ejbRemove() throws RemoteException {

  }

  public void setSessionContext(SessionContext arg0) throws RemoteException {
    this.ctx = arg0;
  }
}


