package org.ow2.bonita.facade.ejb.ejb2;

import java.lang.reflect.Proxy;
import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.rmi.PortableRemoteObject;

import org.ow2.bonita.facade.APIAccessor;
import org.ow2.bonita.facade.CommandAPI;
import org.ow2.bonita.facade.DefinitionAPI;
import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.RuntimeAPI;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2CommandAPIHome;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2DefinitionAPIHome;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2ManagementAPIHome;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2RuntimeAPIHome;
import org.ow2.bonita.facade.exception.BonitaInternalException;
import org.ow2.bonita.facade.remote.RemoteCommandAPI;
import org.ow2.bonita.facade.remote.RemoteDefinitionAPI;
import org.ow2.bonita.facade.remote.RemoteManagementAPI;
import org.ow2.bonita.facade.remote.RemoteRuntimeAPI;
import org.ow2.bonita.util.AccessorUtil;

public class EJB2ClientAPIAccessorImpl extends EJB2QueryAPIAccessorImpl implements APIAccessor {

  private RemoteCommandAPI commandAPI;
  private RemoteDefinitionAPI definitionAPI;
  private RemoteManagementAPI managementAPI;
  private RemoteRuntimeAPI runtimeAPI;
  
  public CommandAPI getCommandAPI() {
    if (commandAPI == null) {
      EJB2CommandAPIHome commandAPIHome = (EJB2CommandAPIHome) PortableRemoteObject.narrow(
           AccessorUtil.<Object>lookup(AccessorUtil.COMMANDAPI_JNDINAME), EJB2CommandAPIHome.class);
      try {
        commandAPI = commandAPIHome.create();
      } catch (RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    Class<CommandAPI> clazz = CommandAPI.class;
    EJB2ClientAPIInterceptor apiInterceptor = new EJB2ClientAPIInterceptor(commandAPI);
    return (CommandAPI) Proxy.newProxyInstance(APIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  }

  public DefinitionAPI getDefinitionAPI() {
    if (definitionAPI == null) {
      EJB2DefinitionAPIHome definitionAPIHome = (EJB2DefinitionAPIHome) PortableRemoteObject.narrow(
         AccessorUtil.<Object>lookup(AccessorUtil.DEFINITIONAPI_JNDINAME), EJB2DefinitionAPIHome.class);
      try {
        definitionAPI = definitionAPIHome.create();
      } catch (RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    Class<DefinitionAPI> clazz = DefinitionAPI.class;
    EJB2ClientAPIInterceptor apiInterceptor = new EJB2ClientAPIInterceptor(definitionAPI);
    return (DefinitionAPI) Proxy.newProxyInstance(APIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  }

  public ManagementAPI getManagementAPI() {
    if (managementAPI == null) {
      EJB2ManagementAPIHome managementAPIHome = (EJB2ManagementAPIHome) PortableRemoteObject.narrow(
        AccessorUtil.<Object>lookup(AccessorUtil.MANAGEMENT_JNDINAME), EJB2ManagementAPIHome.class);
      try {
        managementAPI = managementAPIHome.create();
      } catch (RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    Class<ManagementAPI> clazz = ManagementAPI.class;
    EJB2ClientAPIInterceptor apiInterceptor = new EJB2ClientAPIInterceptor(managementAPI);
    return (ManagementAPI) Proxy.newProxyInstance(APIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  }

  public RuntimeAPI getRuntimeAPI() {
    if (runtimeAPI == null) {
      EJB2RuntimeAPIHome runtimeAPIHome = (EJB2RuntimeAPIHome) PortableRemoteObject.narrow(
        AccessorUtil.<Object>lookup(AccessorUtil.RUNTIMEAPI_JNDINAME), EJB2RuntimeAPIHome.class);
      try {
        runtimeAPI = runtimeAPIHome.create();
      } catch (RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    Class<RuntimeAPI> clazz = RuntimeAPI.class;
    EJB2ClientAPIInterceptor apiInterceptor = new EJB2ClientAPIInterceptor(runtimeAPI);
    return (RuntimeAPI) Proxy.newProxyInstance(APIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  }

}
