package org.ow2.bonita.facade.ejb.ejb2;

import java.lang.reflect.Proxy;
import java.rmi.RemoteException;

import javax.ejb.CreateException;
import javax.rmi.PortableRemoteObject;

import org.ow2.bonita.facade.QueryAPIAccessor;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.QueryRuntimeAPI;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2QueryDefinitionAPIHome;
import org.ow2.bonita.facade.ejb.ejb2.home.EJB2QueryRuntimeAPIHome;
import org.ow2.bonita.facade.exception.BonitaInternalException;
import org.ow2.bonita.facade.impl.QueryAPIAccessorImpl;
import org.ow2.bonita.facade.remote.RemoteQueryDefinitionAPI;
import org.ow2.bonita.facade.remote.RemoteQueryRuntimeAPI;
import org.ow2.bonita.util.AccessorUtil;

public class EJB2QueryAPIAccessorImpl extends QueryAPIAccessorImpl {
  
  private RemoteQueryDefinitionAPI queryDefinitionAPI;
  private RemoteQueryRuntimeAPI queryRuntimeAPI;
  
  public QueryDefinitionAPI getQueryDefinitionAPI() {
    if (queryDefinitionAPI == null) {
      EJB2QueryDefinitionAPIHome queryDefinitionHome = (EJB2QueryDefinitionAPIHome) PortableRemoteObject.narrow(
        AccessorUtil.<Object>lookup(AccessorUtil.QUERYDEFINITION_JNDINAME), EJB2QueryDefinitionAPIHome.class);
      try {
        queryDefinitionAPI = queryDefinitionHome.create();
      } catch (RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    Class<QueryDefinitionAPI> clazz = QueryDefinitionAPI.class;
    EJB2ClientAPIInterceptor apiInterceptor = new EJB2ClientAPIInterceptor(queryDefinitionAPI);
    return (QueryDefinitionAPI) Proxy.newProxyInstance(QueryAPIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  }
  
  public QueryRuntimeAPI getQueryRuntimeAPI() {
    if (queryRuntimeAPI == null) {
      EJB2QueryRuntimeAPIHome queryRuntimeAPIHome = (EJB2QueryRuntimeAPIHome) PortableRemoteObject.narrow(
        AccessorUtil.<Object>lookup(AccessorUtil.QUERYRUNTIME_JNDINAME), EJB2QueryRuntimeAPIHome.class);
      try {
        queryRuntimeAPI = queryRuntimeAPIHome.create();
      } catch (RemoteException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      } catch (CreateException e) {
        throw new BonitaInternalException(e.getMessage(), e);
      }
    }
    Class<QueryRuntimeAPI> clazz = QueryRuntimeAPI.class;
    EJB2ClientAPIInterceptor apiInterceptor = new EJB2ClientAPIInterceptor(queryRuntimeAPI);
    return (QueryRuntimeAPI) Proxy.newProxyInstance(QueryAPIAccessor.class.getClassLoader(),
        new Class[] {clazz}, apiInterceptor);
  }
}
