/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.exception;

import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.BonitaException;

/**
 * Exception thrown when the parameters given to a method does not identify clearly an ActivityInstance
 * (more than one ActivityInstance match with the given parameters).
 * This is the case when an xpdl activity is executed more than once (for example in a loop).
 * 
 * Using an ActivityUUID or adding an iterationId to the method can solve this problem.
 * 
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class AmbiguousActivityException extends BonitaException {

  private final ProcessInstanceUUID instanceUUID;
  private final String activityId;

  public AmbiguousActivityException(final ProcessInstanceUUID instanceUUID,
      final String activityId) {
    super("More than one activity " + activityId + " was found in instance " + instanceUUID
        + ". Please specify wanted iterationId.");
    this.instanceUUID = instanceUUID;
    this.activityId = activityId;
  }

  public AmbiguousActivityException(AmbiguousActivityException e) {
    super(e.getMessage());
    this.instanceUUID = e.getInstanceUUID();
    this.activityId = e.getActivityId();
  }

  public static AmbiguousActivityException build(Throwable e) {
    if (!(e instanceof AmbiguousActivityException)) {
      throw new BonitaInternalException("Cannot create a AmbiguousActivityException", e);
    }
    return new AmbiguousActivityException((AmbiguousActivityException)e);
  }

  public String getActivityId() {
    return this.activityId;
  }


  public ProcessInstanceUUID getInstanceUUID() {
    return instanceUUID;
  }


}
