package org.ow2.bonita.facade.exception;

import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.DataFieldDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.util.BonitaException;


public class DataFieldNotFoundException extends BonitaException {

  private final String dataFieldId;
  private final ProcessDefinitionUUID processUUID;
  private final ActivityDefinitionUUID activityUUID;
  private final DataFieldDefinitionUUID dataFieldUUID;

  public DataFieldNotFoundException(final String dataFieldId, final ProcessDefinitionUUID processDefinitionUUID) {
    super("Can't find a dataField with id " + dataFieldId + " in process " + processDefinitionUUID);
    this.dataFieldId = dataFieldId;
    this.processUUID = processDefinitionUUID;
    this.activityUUID = null;
    this.dataFieldUUID = null;
  }

  public DataFieldNotFoundException(final String dataFieldId, final ActivityDefinitionUUID activityDefinitionUUID) {
    super("Can't find a dataField with id " + dataFieldId + " in activity " + activityDefinitionUUID);
    this.dataFieldId = dataFieldId;
    this.processUUID = null;
    this.activityUUID = activityDefinitionUUID;
    this.dataFieldUUID = null;
  }

  public DataFieldNotFoundException(final DataFieldDefinitionUUID dataFieldDefinitionUUID) {
    super("Can't find a dataField with uuid " + dataFieldDefinitionUUID);
    this.dataFieldUUID = dataFieldDefinitionUUID;
    this.dataFieldId = null;
    this.processUUID = null;
    this.activityUUID = null;
  }

  public DataFieldNotFoundException(DataFieldNotFoundException e) {
    super(e.getMessage());
    this.dataFieldId = e.getDataFieldId();
    this.processUUID = e.getProcessUUID();
    this.activityUUID = e.getActivityUUID();
    this.dataFieldUUID = e.getDataFieldUUID();
  }

  public static DataFieldNotFoundException build(Throwable e) {
    if (!(e instanceof DataFieldNotFoundException)) {
      throw new BonitaInternalException("Cannot create a DataFieldNotFoundException", e);
    }
    return new DataFieldNotFoundException((DataFieldNotFoundException)e);
  }

  public String getDataFieldId() {
    return this.dataFieldId;
  }
  public ProcessDefinitionUUID getProcessUUID() {
    return processUUID;
  }
  public DataFieldDefinitionUUID getDataFieldUUID() {
    return dataFieldUUID;
  }

  public ActivityDefinitionUUID getActivityUUID() {
    return activityUUID;
  }
}
