/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.exception;

import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.util.BonitaException;

/**
 * This exception is thrown when a user attemps to execute an unauthorized action.
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras.
 */
public class UnAuthorizedUserException extends BonitaException {
  private String instanceUUID;
  private TaskUUID taskUUID;
  private String userId;

  /**
   * Constructs an UnAuthorizedUserException for instance.
   * TaskUUID is set to null.
   * @param msg the detail message.
   * @param instanceUUID the instance processDefinitionUUID.
   * @param userId the user processDefinitionUUID.
   */
  public UnAuthorizedUserException(String msg, String instanceUUID, String userId) {
    super("User : " + userId + " is unauthorized " + msg + " with processUUID = " + instanceUUID);
    this.instanceUUID = instanceUUID;
    this.userId = userId;
    this.taskUUID = null;
  }

  /**
   * Constructs an UnAuthorizedUserException for task.
   * @param msg the detail message.
   * @param instanceUUID the instance processDefinitionUUID.
   * @param taskUUID the task processDefinitionUUID.
   * @param userId the user processDefinitionUUID.
   */
  public UnAuthorizedUserException(String msg, String instanceUUID, TaskUUID taskUUID, String userId) {
    super("User : " + userId + " is unauthorized " + msg + " with taskUUID = "
        + taskUUID + " for instance with processUUID = " + instanceUUID);
    this.instanceUUID = instanceUUID;
    this.taskUUID = taskUUID;
    this.userId = userId;
  }

  public UnAuthorizedUserException(UnAuthorizedUserException e) {
    super(e.getMessage());
    this.instanceUUID = e.getInstanceUUID();
    this.taskUUID = e.getTaskUUID();
    this.userId = e.getUserId();
  }

  public UnAuthorizedUserException(String msg, TaskUUID taskUUID, String userId) {
    super("User : " + userId + " is unauthorized " + msg + " with taskUUID = "
        + taskUUID + " for tasl with uuid = " + taskUUID);
    this.taskUUID = taskUUID;
    this.userId = userId;
  }

  public static UnAuthorizedUserException build(Throwable e) {
    if (!(e instanceof UnAuthorizedUserException)) {
      throw new BonitaInternalException("Cannot create a UnAuthorizedUserException", e);
    }
    return new UnAuthorizedUserException((UnAuthorizedUserException)e);
  }

  public TaskUUID getTaskUUID() {
    return this.taskUUID;
  }

  public String getInstanceUUID() {
    return this.instanceUUID;
  }

  public String getUserId() {
    return this.userId;
  }
}
