/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.exception;

import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.BonitaException;


public class UndeletablePackageException extends BonitaException {

  private PackageDefinitionUUID packageDefinitionUUID;
  private ProcessDefinitionUUID processDefinitionUUID;
  private ProcessInstanceUUID processInstanceUUID;
  
  public UndeletablePackageException(
      PackageDefinitionUUID packageDefinitionUUID,
      ProcessDefinitionUUID processDefinitionUUID,
      ProcessInstanceUUID processInstanceUUID) {
    super("Impossible to delete package with UUID : " + packageDefinitionUUID 
        + " because, the process " + processDefinitionUUID 
        + " has running instances. For example : " + processInstanceUUID);
    this.packageDefinitionUUID = packageDefinitionUUID;
    this.processDefinitionUUID = processDefinitionUUID;
    this.processInstanceUUID = processInstanceUUID;
  }

  
  public PackageDefinitionUUID getPackageDefinitionUUID() {
    return packageDefinitionUUID;
  }

  
  public ProcessDefinitionUUID getProcessDefinitionUUID() {
    return processDefinitionUUID;
  }

  
  public ProcessInstanceUUID getProcessInstanceUUID() {
    return processInstanceUUID;
  }
  
  
}


