/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.exception;

import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.BonitaException;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class VariableAlreadyExistException extends BonitaException {

  private ProcessInstanceUUID instanceUUID;
  private String activityId;
  private String variableId;

  public VariableAlreadyExistException(final ProcessInstanceUUID instanceUUID,
      final String variableId) {
    super("Variable " + variableId + " already exists in instance " + instanceUUID);
    this.instanceUUID = instanceUUID;
    this.variableId = variableId;
  }

  public VariableAlreadyExistException(final ProcessInstanceUUID instanceUUID,
      final String activityId, final String variableId) {
    super("Variable " + variableId + " already exists in activity " + activityId + " in instance " + instanceUUID);
    this.instanceUUID = instanceUUID;
    this.activityId = activityId;
    this.variableId = variableId;
  }

  public VariableAlreadyExistException(VariableAlreadyExistException e) {
    super(e.getMessage());
    this.instanceUUID = e.getInstanceUUID();
    this.activityId = e.getActivityId();
    this.variableId = e.getVariableId();
  }

  public String getActivityId() {
    return this.activityId;
  }

  public String getVariableId() {
    return this.variableId;
  }

  public ProcessInstanceUUID getInstanceUUID() {
    return instanceUUID;
  }

}
