package org.ow2.bonita.facade.remote;

import java.rmi.RemoteException;
import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.DataFieldDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.ParticipantDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.ActivityDefNotFoundException;
import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.DataFieldNotFoundException;
import org.ow2.bonita.facade.exception.PackageNotFoundException;
import org.ow2.bonita.facade.exception.ParticipantNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ParticipantDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;

public interface RemoteQueryDefinitionAPI {

  Set<PackageDefinition> getPackages() throws RemoteException;
  
  Set<PackageDefinition> getPackages(String packageId) throws RemoteException;
  
  PackageDefinition getPackage(PackageDefinitionUUID packageDefinitionUUID) throws RemoteException, PackageNotFoundException;
  
  Set<PackageDefinition> getPackages(PackageDefinition.PackageState state) throws RemoteException;
  
  Set<PackageDefinition> getPackages(String packageId, PackageDefinition.PackageState state) 
    throws RemoteException, PackageNotFoundException;
  
  Set<ProcessDefinition> getPackageProcesses(PackageDefinitionUUID packageDefinitionUUID) 
    throws RemoteException, PackageNotFoundException;
  
  ProcessDefinition getPackageProcess(PackageDefinitionUUID packageDefinitionUUID, String processId) 
    throws RemoteException, PackageNotFoundException, ProcessNotFoundException;
  
  Set<ProcessDefinition> getProcesses() throws RemoteException;
  
  Set<ProcessDefinition> getProcesses(String processId) throws RemoteException;
  
  ProcessDefinition getProcess(ProcessDefinitionUUID processDefinitionUUID) throws RemoteException, ProcessNotFoundException;
  
  Set<ProcessDefinition> getProcesses(ProcessDefinition.ProcessState processState) throws RemoteException;
  
  Set<ProcessDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState) 
    throws RemoteException, ProcessNotFoundException;
  
  Set<ParticipantDefinition> getProcessParticipants(ProcessDefinitionUUID processDefinitionUUID) 
    throws RemoteException, ProcessNotFoundException;
  
  ParticipantDefinition getProcessParticipant(ProcessDefinitionUUID processDefinitionUUID,
      String participantId) throws RemoteException, ProcessNotFoundException, ParticipantNotFoundException;

  Set<ActivityDefinition> getProcessActivities(ProcessDefinitionUUID processDefinitionUUID) 
    throws RemoteException, ProcessNotFoundException;
  
  ActivityDefinition getProcessActivity(ProcessDefinitionUUID processDefinitionUUID, String activityId) 
    throws RemoteException, ProcessNotFoundException, ActivityNotFoundException;
  
  ActivityDefinitionUUID getProcessActivityId(ProcessDefinitionUUID processDefinitionUUID,
      String activityName) throws RemoteException, ProcessNotFoundException;

  ParticipantDefinitionUUID getProcessParticipantId(ProcessDefinitionUUID processDefinitionUUID,
      String participantName) throws RemoteException, ProcessNotFoundException;
  
  PackageDefinition getLastPackage(String packageId) throws RemoteException, PackageNotFoundException;
  
  ProcessDefinition getLastProcess(String processId) throws RemoteException, ProcessNotFoundException;

  Set<DataFieldDefinition> getProcessDataFields(ProcessDefinitionUUID processDefinitionUUID)
    throws RemoteException, ProcessNotFoundException;

  DataFieldDefinition getProcessDataField(ProcessDefinitionUUID processDefinitionUUID, String dataFieldId)
    throws RemoteException, ProcessNotFoundException, DataFieldNotFoundException;

  Set<DataFieldDefinition> getActivityDataFields(ActivityDefinitionUUID uuid)
    throws RemoteException, ActivityDefNotFoundException;
  
  DataFieldDefinition getActivityDataField(ActivityDefinitionUUID activityDefinitionUUID, String dataFieldId)
    throws RemoteException, ActivityDefNotFoundException, DataFieldNotFoundException;
}
