package org.ow2.bonita.facade.remote;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.facade.exception.ActivityNotFoundException;
import org.ow2.bonita.facade.exception.AmbiguousActivityException;
import org.ow2.bonita.facade.exception.InstanceNotFoundException;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.exception.TaskNotFoundException;
import org.ow2.bonita.facade.exception.VariableNotFoundException;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityInstance;
import org.ow2.bonita.facade.runtime.ProcessInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;

public interface RemoteQueryRuntimeAPI {

  ProcessInstance getProcessInstance(ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException, RemoteException;

  Set<ProcessInstance> getProcessInstances(ProcessDefinitionUUID processUUID) throws ProcessNotFoundException, RemoteException;

  ActivityInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID, String activityId,
      String iterationId, String activityInstanceId) throws InstanceNotFoundException, ActivityNotFoundException, RemoteException;

  ActivityInstance<ActivityBody> getActivityInstance(ActivityInstanceUUID
      activityUUID) throws ActivityNotFoundException, RemoteException;

  ActivityInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID,
      String activityId) throws InstanceNotFoundException,
      ActivityNotFoundException, AmbiguousActivityException, RemoteException;

  Set<ActivityInstance<ActivityBody>> getActivityInstances(
      ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException, RemoteException;

  Set<ActivityInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID,
      String activityId) throws InstanceNotFoundException, ActivityNotFoundException, RemoteException;

  ActivityInstance<TaskInstance> getTask(TaskUUID taskUUID) throws TaskNotFoundException, RemoteException;

  Set<ActivityInstance<TaskInstance>> getTasks(
      ProcessInstanceUUID instanceUUID) throws InstanceNotFoundException, RemoteException;

  Collection<ActivityInstance<TaskInstance>> getTaskList(ProcessInstanceUUID instanceUUID, TaskState taskState)
    throws InstanceNotFoundException, RemoteException;

  Collection<ActivityInstance<TaskInstance>> getTaskList(TaskState taskState) throws RemoteException;

  Object getActivityInstanceVariable(ActivityInstanceUUID activityUUID,
      String variableId) throws ActivityNotFoundException, VariableNotFoundException, RemoteException;

  Object getActivityInstanceVariable(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId,
      String variableId) throws InstanceNotFoundException, ActivityNotFoundException, VariableNotFoundException, RemoteException;

  Object getActivityInstanceVariable(ProcessInstanceUUID instanceUUID,
      String activityId, String variableId) throws InstanceNotFoundException,
  ActivityNotFoundException, AmbiguousActivityException, VariableNotFoundException, RemoteException;

  Map<String, Object> getActivityInstanceVariables(ProcessInstanceUUID instanceUUID,
      String activityId)throws ActivityNotFoundException,
      AmbiguousActivityException, InstanceNotFoundException, RemoteException;

  Map<String, Object> getActivityInstanceVariables(ActivityInstanceUUID
      activityUUID)throws ActivityNotFoundException, RemoteException;

  Map<String, Object> getActivityInstanceVariables(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId)
    throws ActivityNotFoundException, InstanceNotFoundException, RemoteException;

  Object getProcessInstanceVariable(ProcessInstanceUUID instanceUUID,
      String variableId) throws InstanceNotFoundException, VariableNotFoundException, RemoteException;

  Map<String, Object> getProcessInstanceVariables(ProcessInstanceUUID
      instanceUUID) throws InstanceNotFoundException, RemoteException;

  Object getVariable(ProcessInstanceUUID instanceUUID, String activityId,
      String variableId) throws InstanceNotFoundException, 
      ActivityNotFoundException, VariableNotFoundException, RemoteException, AmbiguousActivityException;
  Map<String, Object> getVariables(ProcessInstanceUUID instanceUUID, String activityId)
    throws InstanceNotFoundException, ActivityNotFoundException, RemoteException, AmbiguousActivityException;

}
