/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import org.ow2.bonita.facade.runtime.AssignUpdate;
import org.ow2.bonita.facade.runtime.TaskState;


public class AssignUpdateImpl extends UpdateImpl implements AssignUpdate  {

  protected Set<String> candidates;
  protected String userId;

  protected AssignUpdateImpl() { }

  public AssignUpdateImpl(Date date, TaskState taskState, String assignerUserId,
      Set<String> candidates, String userId) {
    super();
    this.date = date;
    this.state = taskState;
    this.updateUserId = assignerUserId;
    if (candidates != null && !candidates.isEmpty()) {
      this.candidates = new HashSet<String>();
      for (String candidate : candidates) {
        this.candidates.add(candidate);
      }
    }
    this.userId = userId;
  }

  public AssignUpdateImpl(AssignUpdate assignUpdate) {
    this.date = assignUpdate.getUpdatedDate();
    this.state = assignUpdate.getTaskState();
    this.updateUserId = assignUpdate.getUpdatedBy();
    Set<String> candidates = assignUpdate.getCandidates();
    if (candidates != null && !candidates.isEmpty()) {
      this.candidates = new HashSet<String>();
      for (String candidate : candidates) {
        this.candidates.add(candidate);
      }
    }
    this.userId = assignUpdate.getUserId();
  }

  public String getUserId() {
    return userId;
  }

  public Set<String> getCandidates() {
    return candidates;
  }
}


