/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.util.Date;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.jbpm.pvm.internal.type.Variable;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.VariableUpdate;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.Tool;
import org.ow2.novabpm.util.Misc;

/**
 * @author Pierre Vigneras
 * @date Feb 5, 2008
 */
public class ProcessFullInstanceImpl extends ProcessInstanceImpl implements ProcessFullInstance {

  protected long dbid;

  protected Map<String, Variable> persistableInitialVariableValues;

  protected Set<ActivityFullInstance<ActivityBody>> activities;


  protected ProcessFullInstanceImpl() {
    super();
  }

  public ProcessFullInstanceImpl(final ProcessInstanceUUID recordUUID, final PackageDefinitionUUID packageUUID,
      final ProcessDefinitionUUID processUUID,
      final ProcessInstanceUUID parentInstanceUUID, final Map<String, Object> initialVariableValues) {
    super(recordUUID, packageUUID, processUUID, parentInstanceUUID, null);

    this.persistableInitialVariableValues =
      Tool.createVariableMap(initialVariableValues);
  }

  public ProcessFullInstanceImpl(final ProcessFullInstance src) {
    super(src);
    this.initialVariableValues = null;
    this.persistableInitialVariableValues = Tool.createVariableMap(src.getInitialVariableValues());
    Set<ActivityFullInstance<ActivityBody>> acts = src.getActivities();
    if (acts != null) {
      this.activities = new HashSet<ActivityFullInstance<ActivityBody>>();
      for (ActivityFullInstance<ActivityBody> activity : acts) {
        this.activities.add(new ActivityFullInstanceImpl<ActivityBody>(activity));
      }
    }
  }

  public void addActivity(ActivityFullInstance<ActivityBody> activity) {
    if (getActivities() == null)  {
      activities = new HashSet<ActivityFullInstance<ActivityBody>>();
    }
    activities.add(activity);
  }

  public Set<ActivityFullInstance<ActivityBody>> getActivities() {
    return activities;
  }

  public Set<ActivityFullInstance<ActivityBody>> getActivities(String activityId) {
    Set<ActivityFullInstance<ActivityBody>> matchingActivities = new HashSet<ActivityFullInstance<ActivityBody>>();
    if (getActivities() != null) {
      for (ActivityFullInstance<ActivityBody> activity : getActivities()) {
        if (activity.getActivityId().equals(activityId)) {
          matchingActivities.add(activity);
        }
      }
    }
    return matchingActivities;
  }

  public ActivityFullInstance<ActivityBody> getActivity(String activityId,
      String iterationId, String activityInstanceId) {
    if (getActivities() != null) {
      for (ActivityFullInstance<ActivityBody> activity : getActivities()) {
        if (activity.getActivityId().equals(activityId)
            &&
            (
                (activity.getIterationId() == null && iterationId == null)
                ||
                activity.getIterationId().equals(iterationId)
            )
            &&
            (
                (activity.getActivityInstanceId() == null && activityInstanceId == null)
                ||
                activity.getActivityInstanceId().equals(activityInstanceId)
            )
        ) {
          return activity;
        }
      }
    }
    return null;
  }

  public InstanceState getInstanceState() {
    if (getEndedBy() != null) {
      return InstanceState.FINISHED;
    }
    if (getStartedBy() != null) {
      return InstanceState.STARTED;
    }
    return InstanceState.INITIAL;
  }

  public void setStartedBy(final String userId) {
    Misc.badStateIfNotNull(getStartedBy(), "startedBy can not be set twice!");
    this.startedBy = userId;
  }

  public void setStartedDate(final Date date) {
    Misc.badStateIfNotNull(getStartedDate(), "startedDate can not be set twice!");
    this.startedDate = date;
  }

  public void setEndedBy(final String userId) {
    Misc.badStateIfNotNull(getEndedBy(), "endedBy can not be set twice!");
    this.endedBy = userId;
  }

  public void setEndedDate(final Date date) {
    Misc.badStateIfNotNull(getEndedDate(), "endedDate can not be set twice!");
    this.endedDate = date;
  }

  public Map<String, Object> getInitialVariableValues() {
    return Tool.getVariableValues(persistableInitialVariableValues);
  }
  
  public void addVariableUpdate(VariableUpdate varUpdate) {
    variableUpdates.add(varUpdate);
  }
/*
  public void end() {
    if (this.activities != null) {
      for (ActivityFullInstance<ActivityBody> activity : activities) {
        ActivityBody body = activity.getBody(); 
        if (body instanceof TaskFullInstance) {
          ((TaskFullInstance)body).setTaskRuntime(null);
        }
      }
    }
  }
  */
}
