/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.facade.runtime.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.Set;

import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.TaskFullInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.runtime.Update;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.novabpm.util.Misc;


/**
 * @author Guillaume Porcher
 *
 */
public class TaskFullInstanceImpl extends TaskInstanceImpl implements TaskFullInstance {

  protected TaskFullInstanceImpl() {
  }

  public TaskFullInstanceImpl(TaskUUID taskUUID, TaskState state, Date createdDate, Date dueDate) {
    super(taskUUID, null, null, state, createdDate, dueDate, null);
  }

  public TaskFullInstanceImpl(TaskFullInstanceImpl task) {
    super(task);
  }

  @Override
  public ActivityBody copy() {
    return new TaskFullInstanceImpl(this);
  }

  public void setStartedBy(String userId) {
    Misc.badStateIfNotNull(getStartedBy(), "startedBy can not be set twice!");
    this.startedBy = userId;
  }

  public void setInitialUserId(String userId) {
    //Misc.badStateIfTrue(userIdInit, "userId can not be set twice!");
   // Misc.badStateIfFalse(candidatesInit, "userId can not be set before candidates!");
    //this.userIdInit = true;
    this.initialUserId = userId;
  }

  public void setInitialCandidates(Set<String> candidates) {
    //Misc.badStateIfTrue(candidatesInit, "candidates can not be set twice!");
    //Misc.badStateIfTrue(userId != null, "candidates can not be set after userId!");
    this.initialCandidates = candidates;
    //this.candidatesInit = true;
  }

  public void setEndedBy(String userId) {
    Misc.badStateIfNotNull(getEndedBy(), "endedBy can not be set twice!");
    Misc.badStateIfNull(getStartedBy(), "endedBy can not be set before startedBy!");
    this.endedBy = userId;
  }

  public void setTaskState(TaskState newState, String userId) {
    TaskState oldState;
    if (getUpdates() == null) {
      this.updates = new ArrayList<Update>();
      oldState = TaskState.INITIAL;
    } else {
      //get the old task state from the previous update
      oldState = this.updates.get(updates.size() - 1).getTaskState();
    }
    //add a state update
    this.updates.add(new StateUpdateImpl(new Date(), newState, oldState, userId));
  }

  public void setTaskAssign(TaskState taskState, String assignerUserId, Set<String> candidates, String userId) {
    if (getUpdates() == null) {
      this.updates = new ArrayList<Update>();
    }
    this.updates.add(new AssignUpdateImpl(new Date(), taskState, assignerUserId, candidates, userId));
  }

  public long getDbid() {
    return dbid;
  }
  /*
  public void setTaskRuntime(Object o) {
    //todo
  }
   */
}
