/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.parsing.binding;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jbpm.pvm.internal.util.XmlUtil;
import org.jbpm.pvm.internal.xml.Parse;
import org.jbpm.pvm.internal.xml.Parser;
import org.ow2.bonita.facade.def.dataType.BasicTypeDefinition;
import org.ow2.bonita.facade.def.dataType.DataTypeDefinition;
import org.ow2.bonita.facade.def.dataType.EnumerationTypeDefinition;
import org.ow2.bonita.facade.def.dataType.impl.BasicTypeDefinitionImpl;
import org.ow2.bonita.facade.def.dataType.impl.DataTypeDefinitionImpl;
import org.ow2.bonita.facade.def.dataType.impl.EnumerationTypeDefinitionImpl;
import org.ow2.bonita.parsing.XpdlParser;
import org.w3c.dom.Element;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class DataTypeBinding extends MajorElementBinding {

  public DataTypeBinding() {
    super("DataType");
    category = XpdlParser.CATEGORY_DATATYPES;
  }
  
  public Object parse(Element dataTypeElement, Parse parse, Parser parser) {
    Element realDataTypeElement = XmlUtil.element(dataTypeElement);
    DataTypeDefinition dataTypeDefinition = null;
    if (realDataTypeElement.getLocalName().equals("BasicType")) {
      dataTypeDefinition = parseBasicType(realDataTypeElement, parse);
    } else if (realDataTypeElement.getLocalName().equals("DeclaredType")) {
      dataTypeDefinition = parseDeclaredType(realDataTypeElement, parse);
    } else if (realDataTypeElement.getLocalName().equals("SchemaType")) {
      dataTypeDefinition = parseSchemaType(realDataTypeElement, parse);
    } else if (realDataTypeElement.getLocalName().equals("ExternalReference")) {
      dataTypeDefinition = parseExternalReference(realDataTypeElement, parse);
    } else if (realDataTypeElement.getLocalName().equals("RecordType")) {
      dataTypeDefinition = parseRecordType(realDataTypeElement, parse);
    } else if (realDataTypeElement.getLocalName().equals("UnionType")) {
      dataTypeDefinition = parseUnionType(realDataTypeElement, parse);
    } else if (realDataTypeElement.getLocalName().equals("EnumerationType")) {
      dataTypeDefinition = parseEnumerationType(realDataTypeElement, parse);
    } else if (realDataTypeElement.getLocalName().equals("ArrayType")) {
      dataTypeDefinition = parseArrayType(realDataTypeElement, parse);
    } else if (realDataTypeElement.getLocalName().equals("ListType")) {
      dataTypeDefinition = parseListType(realDataTypeElement, parse);
    }
    return dataTypeDefinition;
  }

  private DataTypeDefinition parseListType(Element dataTypeElement, Parse parse) {
    parse.addProblem("Unsupported DataType: " + dataTypeElement.getLocalName());
    return null;
  }

  private DataTypeDefinition parseArrayType(Element dataTypeElement, Parse parse) {
    parse.addProblem("Unsupported DataType: " + dataTypeElement.getLocalName());
    return null;
  }

  private DataTypeDefinition parseEnumerationType(Element dataTypeElement, Parse parse) {
    Set<String> enuemarionValues = new HashSet<String>();
    List<Element> enumerationValueElements = XmlUtil.elements(dataTypeElement, "EnumerationValue");
    if (enumerationValueElements == null) {
      parse.addProblem("No enumeration specified in EnumerationType. Please specify at least one");
    } else {
      for (Element enumeratiuonValueElement : enumerationValueElements) {
        enuemarionValues.add(XmlUtil.attribute(enumeratiuonValueElement, "Name"));
      }
    }
    EnumerationTypeDefinition  enumerationType = new EnumerationTypeDefinitionImpl(enuemarionValues);
    return new DataTypeDefinitionImpl(DataTypeDefinition.Type.EnumerationType, enumerationType);
  }

  private DataTypeDefinition parseUnionType(Element dataTypeElement, Parse parse) {
    parse.addProblem("Unsupported DataType: " + dataTypeElement.getLocalName());
    return null;
  }

  private DataTypeDefinition parseRecordType(Element dataTypeElement, Parse parse) {
    parse.addProblem("Unsupported DataType: " + dataTypeElement.getLocalName());
    return null;
  }

  private DataTypeDefinition parseExternalReference(Element dataTypeElement, Parse parse) {
    parse.addProblem("Unsupported DataType: " + dataTypeElement.getLocalName());
    return null;
  }

  private DataTypeDefinition parseSchemaType(Element dataTypeElement, Parse parse) {
    parse.addProblem("Unsupported DataType: " + dataTypeElement.getLocalName());
    return null;
  }

  private DataTypeDefinition parseDeclaredType(Element dataTypeElement, Parse parse) {
    parse.addProblem("Unsupported DataType: " + dataTypeElement.getLocalName());
    return null;
  }

  public DataTypeDefinition parseBasicType(Element dataTypeElement, Parse parse) {
    BasicTypeDefinition.Type type = getEnumValue(org.ow2.bonita.facade.def.dataType.BasicTypeDefinition.Type.class,
        XmlUtil.attribute(dataTypeElement, "Type"), null);
    BasicTypeDefinition basicType = new BasicTypeDefinitionImpl(type);

    // STRING, FLOAT, INTEGER, BOOLEAN, DATETIME are supported
    if (BasicTypeDefinition.Type.REFERENCE.equals(basicType.getType())) {
      parse.addProblem("Unsupported BasicType Type:  " + basicType.getType());
    }
    return new DataTypeDefinitionImpl(DataTypeDefinition.Type.BasicType, basicType);
  }
}


