package org.ow2.bonita.persistence;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.jbpm.pvm.job.Timer;
import org.jbpm.pvm.session.PvmDbSession;
import org.ow2.bonita.definition.GlobalClassData;
import org.ow2.bonita.definition.PackageClassData;
import org.ow2.bonita.definition.PackageDependency;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.runtime.ActivityInstanceKey;
import org.ow2.bonita.runtime.ProcessInstanceKey;
import org.ow2.bonita.runtime.TaskRunTime;
import org.ow2.bonita.runtime.XpdlExecution;
import org.ow2.bonita.runtime.XpdlInstance;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public interface XpdlPersistenceService extends PvmDbSession {

  
  XpdlInstance getXpdlInstance(ProcessInstanceUUID instanceUUID);

  List<XpdlInstance> getXpdlInstances(ProcessDefinitionUUID processId);

  XpdlProcess findXpdlProcessById(ProcessDefinitionUUID processId);

  List<XpdlProcess> getXpdlProcesses();

  List<TaskRunTime> getInstanceTasks(ProcessInstanceUUID instanceUUID);

  Set<ActivityFullInstance<TaskInstance>> getUserTasks(String userId, TaskState taskState);
  Set<ActivityFullInstance<TaskInstance>> getUserInstanceTasks(String userId, 
      ProcessInstanceUUID instanceUUID, TaskState taskState);

  boolean hasProcessWithUUID(ProcessDefinitionUUID uuid);
  boolean hasPackageWithUUID(PackageDefinitionUUID uuid);

  PackageClassData getPackageClassData(PackageDefinitionUUID processId);
  GlobalClassData getGlobalClassData(String className);

  ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID);
  Set<ProcessFullInstance> getProcessInstances(ProcessDefinitionUUID processId);

  ActivityFullInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId);
  Set<ActivityFullInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID);

  ActivityFullInstance< TaskInstance > getTaskInstance(TaskUUID taskUUID);
  Set<ActivityFullInstance< TaskInstance >> getTaskInstances(ProcessInstanceUUID instanceUUID);

  TaskRunTime getTask(TaskUUID taskUUID);
  /**
   * @param processUUID
   * @param nodeName
   * @return
   */
  XpdlExecution getExecutionPointingOnNode(ProcessInstanceUUID instanceUUID, String nodeName);

  /**
   * @param packageDefinitionUUID
   * @return
   */
  Set<PackageDependency> getPackageDependencies(String className);
  PackageDependency getPackageDependency(PackageDefinitionUUID packageId);

  /*
   * DEFINITION
   */
  Set<PackageFullDefinition> getPackages();
  Set<PackageFullDefinition> getPackages(String packageId);
  PackageFullDefinition getPackage(PackageDefinitionUUID packageId);
  Set<PackageFullDefinition> getPackages(PackageDefinition.PackageState packageState);
  Set<PackageFullDefinition> getPackages(String packageId, PackageDefinition.PackageState packageState);
  PackageFullDefinition getLastDeployedPackage(String packageId);


  Set<ProcessFullDefinition> getPackageProcesses(PackageDefinitionUUID packageId);
  Set<ProcessFullDefinition> getProcesses();
  Set<ProcessFullDefinition> getProcesses(String processId);
  Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState);
  Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState);
  ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID);
  ProcessFullDefinition getLastDeployedProcess(String processId);

  Set<String> getDependentPackages(PackageDefinitionUUID packageUUID);

  List<Timer> getInstanceTimers(ProcessInstanceUUID instanceUUID);

  ActivityFullInstance<ActivityBody> getActivityInstance(
      ActivityInstanceUUID activityInstanceUUID);

  ProcessFullDefinition getPackageProcess(PackageDefinitionUUID packageDefinitionUUID,
      String processId);

  XpdlProcess findLatestProcessById(String subProcessId);

  ProcessInstanceKey getProcessInstanceKey(ProcessDefinitionUUID processUUID);

  ActivityInstanceKey getActivityInstanceKey(ProcessInstanceUUID instanceUUID,
      String activityName);

  Collection<Timer> getExecutionTimers(XpdlExecution xpdlExecution);

  ActivityDefinition getActivityDefinition(ActivityDefinitionUUID activityDefinitionUUID);
}
