package org.ow2.bonita.persistence.db;

import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.def.majorElement.impl.PackageFullDefinitionImpl;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.runtime.impl.ProcessFullInstanceImpl;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.persistence.XpdlPersistenceService;
import org.ow2.bonita.services.impl.AbstractHistory;
import org.ow2.bonita.util.EnvTool;
import org.ow2.novabpm.util.Misc;

/**
 * 
 * @author Thomas Gueze
 *
 */
public class DbHistory extends AbstractHistory {

  private String persistenceServiceName;
  
  private XpdlPersistenceService getPersistenceService() {
    return EnvTool.getPersistenceService(persistenceServiceName);
  }
  
  public DbHistory(String name) {
    this.persistenceServiceName = name;
  }
  
  public void archive(PackageFullDefinition packageDef) {
    Misc.checkArgsNotNull(packageDef);
    if (getPersistenceService().getPackage(packageDef.getPackageDefinitionUUID()) == null) {
      getPersistenceService().save(new PackageFullDefinitionImpl(packageDef));
    } else {
      throw new IllegalArgumentException(" Can't archive: " + packageDef
          + " --- A record with processDefinitionUUID: "
          + packageDef + " has already been archived ");   
    }
  }

  public void archive(ProcessFullInstance processInst) {
    Misc.checkArgsNotNull(processInst);
    Set<ProcessFullInstance> processInstances = getPersistenceService()
      .getProcessInstances(processInst.getProcessDefinitionUUID());
    if (!processInstances.contains(processInst)) {
      getPersistenceService().save(new ProcessFullInstanceImpl(processInst));
    } else {
      throw new IllegalArgumentException(" Can't archive: " + processInst
          + " --- A record with processDefinitionUUID: "
          + processInst + " has already been archived ");
    }
  }

  public ActivityFullInstance<ActivityBody> getActivityInstance(
    ActivityInstanceUUID activityInstanceUUID) {
    Misc.checkArgsNotNull(activityInstanceUUID);
    return getPersistenceService().getActivityInstance(activityInstanceUUID);
  }

  public PackageFullDefinition getPackage(PackageDefinitionUUID packageUUID) {
    Misc.checkArgsNotNull(packageUUID);
    return getPersistenceService().getPackage(packageUUID);
  }

  public Set<PackageFullDefinition> getPackages() {
    return getPersistenceService().getPackages();
  }

  public ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID) {
    Misc.checkArgsNotNull(instanceUUID);
    return getPersistenceService().getProcessInstance(instanceUUID);
  }

  public Set<ProcessFullInstance> getProcessInstances(
    ProcessDefinitionUUID processUUID) {
    Misc.checkArgsNotNull(processUUID);
    return getPersistenceService().getProcessInstances(processUUID);
  }

  public ActivityFullInstance<TaskInstance> getTaskInstance(TaskUUID taskUUID) {
    Misc.checkArgsNotNull(taskUUID);
    return getPersistenceService().getTaskInstance(taskUUID);
  }

  public Set<ActivityFullInstance<TaskInstance>> getUserInstanceTasks(
    String userId, ProcessInstanceUUID instanceUUID, TaskState taskState) {
    Misc.checkArgsNotNull(userId, instanceUUID, taskState);
    return getPersistenceService().getUserInstanceTasks(userId, instanceUUID, taskState);
  }

  public Set<ActivityFullInstance<TaskInstance>> getUserTasks(String userId,
    TaskState taskState) {
    Misc.checkArgsNotNull(userId, taskState);
    return getPersistenceService().getUserTasks(userId, taskState);
  }
  
  private boolean removeRecord(Object record) {
    getPersistenceService().delete(record);
    return true;
  }
  
  public void remove(PackageFullDefinition package1) {
    removeRecord(package1);
  }
  
  public void remove(ProcessFullInstance processInst) {
    removeRecord(processInst);
  }

  public void clear() {
    Set<PackageFullDefinition> packages = getPersistenceService().getPackages();
    for (PackageFullDefinition pack : packages) {
      getPersistenceService().delete(pack);
    }
    Set<ProcessFullDefinition> processes = getPersistenceService().getProcesses();
    Set<ProcessFullInstance> processInstances;
    for (ProcessFullDefinition process : processes) {
      processInstances = getPersistenceService().getProcessInstances(process.getProcessDefinitionUUID());
      for (ProcessFullInstance processInstance : processInstances) {
        getPersistenceService().delete(processInstance);
      }
    }
  }
}
