/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.persistence.db;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.ow2.bonita.definition.PackageDependency;
import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.runtime.impl.ProcessFullInstanceImpl;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.persistence.XpdlPersistenceService;
import org.ow2.bonita.services.Querier;
import org.ow2.bonita.services.impl.AbstractJournal;
import org.ow2.bonita.util.EnvTool;
import org.ow2.novabpm.util.Misc;

/**
 * @author Guillaume Porcher
 *
 */
public class DbJournal extends AbstractJournal {

  private JournalBuffer buffer = new JournalBuffer();

  private static class JournalBuffer implements Querier {

    public static final String CURRENT_INSTANCE_KEY = "current-xpdl-instance";

    private Map<ProcessInstanceUUID, ProcessFullInstance> processInstances =
      new HashMap<ProcessInstanceUUID, ProcessFullInstance>();

    public void addProcessInstance(ProcessFullInstance instance) {
      processInstances.put(instance.getUUID(), instance);
    }

    public void removeProcessInstance(ProcessFullInstance instance) {
      processInstances.remove(instance.getUUID());
    }

    public ActivityFullInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID,
        String activityId, String iterationId, String activityInstanceId) {
      ProcessFullInstance processFullInstance = processInstances.get(instanceUUID);
      if (processFullInstance == null) {
        return null;
      }

      return processFullInstance.getActivity(activityId, iterationId, activityInstanceId);
    }

    public ActivityFullInstance<ActivityBody> getActivityInstance(ActivityInstanceUUID activityInstanceUUID) {
      for (ProcessFullInstance processFullInstance : processInstances.values()) {
        for (ActivityFullInstance<ActivityBody> activity : processFullInstance.getActivities()) {
          if (activity.getUUID().equals(activityInstanceUUID)) {
            return activity;
          }
        }
      }
      return null;
    }

    public Set<ActivityFullInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID) {
      ProcessFullInstance processFullInstance = processInstances.get(instanceUUID);
      if (processFullInstance == null) {
        return new HashSet<ActivityFullInstance<ActivityBody>>();
      }
      return processFullInstance.getActivities();
    }

    public Set<String> getDependentPackages(PackageDefinitionUUID packageUUID) {
      return new HashSet<String>();
    }

    public PackageFullDefinition getLastDeployedPackage(String packageId) {
      return null;
    }

    public ProcessFullDefinition getLastDeployedProcess(String processId) {
      return null;
    }

    public PackageFullDefinition getPackage(PackageDefinitionUUID packageUUID) {
      return null;
    }

    public ProcessFullDefinition getPackageProcess(PackageDefinitionUUID packageDefinitionUUID,
        String processId) {
      return null;
    }

    public Set<ProcessFullDefinition> getPackageProcesses(PackageDefinitionUUID packageUUID) {
      return new HashSet<ProcessFullDefinition>();
    }

    public Set<PackageFullDefinition> getPackages() {
      return new HashSet<PackageFullDefinition>();
    }

    public Set<PackageFullDefinition> getPackages(String packageId) {
      return new HashSet<PackageFullDefinition>();
    }

    public Set<PackageFullDefinition> getPackages(PackageDefinition.PackageState packageState) {
      return new HashSet<PackageFullDefinition>();
    }

    public Set<PackageFullDefinition> getPackages(String packageId, PackageDefinition.PackageState packageState) {
      return new HashSet<PackageFullDefinition>();
    }

    public ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID) {
      return null;
    }

    public ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID) {
      return processInstances.get(instanceUUID);
    }

    public Set<ProcessFullInstance> getProcessInstances(ProcessDefinitionUUID processUUID) {
      Set<ProcessFullInstance> res = new HashSet<ProcessFullInstance>();
      for (ProcessFullInstance processFullInstance : processInstances.values()) {
        if (processFullInstance.getProcessDefinitionUUID().equals(processUUID)) {
          res.add(processFullInstance);
        }
      }
      return res;
    }

    public Set<ProcessFullDefinition> getProcesses() {
      return new HashSet<ProcessFullDefinition>();
    }

    public Set<ProcessFullDefinition> getProcesses(String processId) {
      return new HashSet<ProcessFullDefinition>();
    }

    public Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState) {
      return new HashSet<ProcessFullDefinition>();
    }

    public Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState) {
      return new HashSet<ProcessFullDefinition>();
    }

    @SuppressWarnings("unchecked")
    public ActivityFullInstance<TaskInstance> getTaskInstance(TaskUUID taskUUID) {
      for (ProcessFullInstance processFullInstance : processInstances.values()) {
        for (ActivityFullInstance<ActivityBody> activity : processFullInstance.getActivities()) {
          ActivityBody body = activity.getBody();
          if ((body instanceof TaskInstance)
              && body.getUUID().equals(taskUUID)) {
            return (ActivityFullInstance) activity;
          }
        }
      }
      return null;
    }


    @SuppressWarnings("unchecked")
    public Set<ActivityFullInstance<TaskInstance>> getTaskInstances(ProcessInstanceUUID instanceUUID) {
      ProcessFullInstance processFullInstance = processInstances.get(instanceUUID);
      if (processFullInstance == null) {
        return new HashSet<ActivityFullInstance<TaskInstance>>();
      }
      Set<ActivityFullInstance<TaskInstance>> res = new HashSet<ActivityFullInstance<TaskInstance>>();
      for (ActivityFullInstance<ActivityBody> activity : processFullInstance.getActivities()) {
        ActivityBody body = activity.getBody();
        if (body instanceof TaskInstance) {
          res.add((ActivityFullInstance) activity);
        }
      }
      return res;
    }

    @SuppressWarnings("unchecked")
    public Set<ActivityFullInstance<TaskInstance>> getUserInstanceTasks(
        String userId, ProcessInstanceUUID instanceUUID, TaskState taskState) {
      ProcessFullInstance processInstance = processInstances.get(instanceUUID);
      Set<ActivityFullInstance<TaskInstance>> result = new HashSet<ActivityFullInstance<TaskInstance>>();
      if (processInstance == null) {
        return result;
      }
      Set<ActivityFullInstance<ActivityBody>> activities = processInstance.getActivities();
      if (activities != null) {
        for (ActivityFullInstance< ? > activity : activities) {
          ActivityBody body = activity.getBody();
          if (body != null && body instanceof TaskInstance) {
            TaskInstance taskInstance = (TaskInstance)body;
            boolean userIdOk = false;
            try {
              String taskUserId = taskInstance.getTaskUser();
              userIdOk = taskUserId != null && taskUserId.equals(userId);
            } catch (IllegalStateException e) {
              // nothing
            }
            boolean candidatesOk = false;
            try {
              Set<String> taskCandidates = taskInstance.getTaskCandidates();
              candidatesOk = taskCandidates != null && taskCandidates.contains(userId);
            } catch (IllegalStateException e) {
              // nothing
            }
            boolean stateOk = taskInstance.getState().equals(taskState);
            if ((userIdOk || candidatesOk) && stateOk) {
              result.add((ActivityFullInstance<TaskInstance>) activity);
            }
          }
        }
      }

      return result;

    }
    public Set<ActivityFullInstance<TaskInstance>> getUserTasks(String userId, TaskState taskState) {
      Set<ActivityFullInstance<TaskInstance>> result = new HashSet<ActivityFullInstance<TaskInstance>>();
      for (ProcessInstanceUUID instanceUUID : processInstances.keySet()) {
        result.addAll(getUserInstanceTasks(userId, instanceUUID, taskState));
      }
      return result;
    }

    public ActivityDefinition getActivity(ActivityDefinitionUUID activityDefinitionUUID) {
      return null;
    }

  }

  private String persistenceServiceName;

  public DbJournal(String persistenceServiceName) {
    this.persistenceServiceName = persistenceServiceName;
  }

  private XpdlPersistenceService getPersistenceService() {
    return EnvTool.getPersistenceService(persistenceServiceName);
  }
  private boolean removeRecord(Object record) {
    getPersistenceService().delete(record);
    return true;
  }

  public void remove(ProcessFullInstance processInst) {
    buffer.removeProcessInstance(processInst);
    removeRecord(processInst);
  }

  public boolean remove(ActivityFullInstance< ? > activityInst) {
    return removeRecord(activityInst);
  }

  @Override
  public void storeInstanceRecord(ProcessFullInstanceImpl instanceRecord) {
    getPersistenceService().save(instanceRecord);
    buffer.addProcessInstance(instanceRecord);
  }


  public ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID) {
    Misc.checkArgsNotNull(instanceUUID);
    ProcessFullInstance processFullInstance = buffer.getProcessInstance(instanceUUID);
    if (processFullInstance != null) {
      return processFullInstance;
    }
    processFullInstance = getPersistenceService().getProcessInstance(instanceUUID);
    if (processFullInstance != null) {
      buffer.addProcessInstance(processFullInstance);
    }
    return processFullInstance;
  }

  public Set<ProcessFullInstance> getProcessInstances(ProcessDefinitionUUID processId) {
    Misc.checkArgsNotNull(processId);

    Set<ProcessFullInstance> processFullInstances = buffer.getProcessInstances(processId);

    Set<ProcessFullInstance> dbInstances = getPersistenceService().getProcessInstances(processId);
    for (ProcessFullInstance instance : dbInstances) {
      buffer.addProcessInstance(instance);
    }
    processFullInstances.addAll(dbInstances);
    return processFullInstances;
  }

  public ActivityFullInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID,
      String activityId, String iterationId, String activityInstanceId) {
    ActivityFullInstance<ActivityBody> activityInstance =
      buffer.getActivityInstance(instanceUUID, activityId, iterationId, activityInstanceId);
    if (activityInstance != null) {
      return activityInstance;
    }
    return getPersistenceService().getActivityInstance(instanceUUID, activityId, iterationId, activityInstanceId);
  }


  public Set<ActivityFullInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID) {
    Set<ActivityFullInstance<ActivityBody>> activityInstances = buffer.getActivityInstances(instanceUUID);
    activityInstances.addAll(getPersistenceService().getActivityInstances(instanceUUID));
    return activityInstances;
  }

  public ActivityFullInstance< TaskInstance > getTaskInstance(TaskUUID taskUUID) {
    ActivityFullInstance<TaskInstance> activityInstance = buffer.getTaskInstance(taskUUID);
    if (activityInstance != null) {
      return activityInstance;
    }
    return getPersistenceService().getTaskInstance(taskUUID);
  }

  public Set<ActivityFullInstance< TaskInstance >> getTaskInstances(ProcessInstanceUUID instanceUUID) {
    Set<ActivityFullInstance<TaskInstance>> activityInstances = buffer.getTaskInstances(instanceUUID);
    activityInstances.addAll(getPersistenceService().getTaskInstances(instanceUUID));
    return activityInstances;
  }

  @Override
  public void storePackageDef(PackageFullDefinition packageDef) {
    getPersistenceService().save(packageDef);
  }
  public void remove(PackageFullDefinition package1) {
    removeRecord(package1);
  }

  public Set<PackageFullDefinition> getPackages() {
    return getPersistenceService().getPackages();
  }

  public Set<PackageFullDefinition> getPackages(String packageId) {
    return getPersistenceService().getPackages(packageId);
  }
  public PackageFullDefinition getPackage(PackageDefinitionUUID packageId) {
    return getPersistenceService().getPackage(packageId);
  }
  public Set<PackageFullDefinition> getPackages(PackageDefinition.PackageState packageState) {
    return getPersistenceService().getPackages(packageState);
  }
  public Set<PackageFullDefinition> getPackages(String packageId, PackageDefinition.PackageState packageState) {
    return getPersistenceService().getPackages(packageId, packageState);
  }
  public PackageFullDefinition getLastDeployedPackage(String packageId) {
    return getPersistenceService().getLastDeployedPackage(packageId);
  }

  public Set<ProcessFullDefinition> getPackageProcesses(PackageDefinitionUUID packageId) {
    return getPersistenceService().getPackageProcesses(packageId);
  }
  public Set<ProcessFullDefinition> getProcesses() {
    return getPersistenceService().getProcesses();
  }
  public Set<ProcessFullDefinition> getProcesses(String processId) {
    return getPersistenceService().getProcesses(processId);
  }
  public Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState) {
    return getPersistenceService().getProcesses(processState);
  }
  public Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState) {
    return getPersistenceService().getProcesses(processId, processState);
  }
  public ProcessFullDefinition getProcess(ProcessDefinitionUUID processId) {
    return getPersistenceService().getProcess(processId);
  }
  public ProcessFullDefinition getLastDeployedProcess(String processId) {
    return getPersistenceService().getLastDeployedProcess(processId);
  }

  public Set<String> getDependentPackages(PackageDefinitionUUID packageId) {
    return getPersistenceService().getDependentPackages(packageId);
  }

  public Set<PackageDependency> getPackageDependencies(String className) {
    return getPersistenceService().getPackageDependencies(className);
  }

  public ActivityFullInstance<ActivityBody> getActivityInstance(
      ActivityInstanceUUID activityInstanceUUID) {
    ActivityFullInstance<ActivityBody> activityInstance = buffer.getActivityInstance(activityInstanceUUID);
    if (activityInstance != null) {
      return activityInstance;
    }
    return getPersistenceService().getActivityInstance(activityInstanceUUID);
  }
  public ProcessFullDefinition getPackageProcess(
      PackageDefinitionUUID packageDefinitionUUID, String processId) {
    return getPersistenceService().getPackageProcess(packageDefinitionUUID, processId);
  }


  public Set<ActivityFullInstance<TaskInstance>> getUserInstanceTasks(String userId,
      ProcessInstanceUUID instanceUUID, TaskState taskState) {
    return getPersistenceService().getUserInstanceTasks(userId, instanceUUID, taskState);
  }
  public Set<ActivityFullInstance<TaskInstance>> getUserTasks(String userId, TaskState taskState) {
    return getPersistenceService().getUserTasks(userId, taskState);
  }

  public void clear() {
    // TODO Auto-generated method stub

  }

  public ActivityDefinition getActivity(ActivityDefinitionUUID activityDefinitionUUID) {
    return getPersistenceService().getActivityDefinition(activityDefinitionUUID);
  }
}
