package org.ow2.bonita.persistence.db;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Logger;

import org.hibernate.FlushMode;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.type.Type;
import org.jbpm.pvm.internal.hibernate.HibernatePvmDbSession;
import org.jbpm.pvm.internal.job.TimerImpl;
import org.jbpm.pvm.job.Timer;
import org.ow2.bonita.definition.GlobalClassData;
import org.ow2.bonita.definition.PackageClassData;
import org.ow2.bonita.definition.PackageDependency;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;
import org.ow2.bonita.persistence.XpdlPersistenceService;
import org.ow2.bonita.runtime.ActivityInstanceKey;
import org.ow2.bonita.runtime.ProcessInstanceKey;
import org.ow2.bonita.runtime.TaskRunTime;
import org.ow2.bonita.runtime.XpdlExecution;
import org.ow2.bonita.runtime.XpdlInstance;
import org.ow2.novabpm.util.hibernate.GenericEnumUserType;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class HibernateXpdlPersistenceService extends
    HibernatePvmDbSession implements XpdlPersistenceService {

  private static final Logger LOG = Logger.getLogger(HibernateXpdlPersistenceService.class.getName());

  private static Type taskStateUserType =
    Hibernate.custom(GenericEnumUserType.class, new String[]{"enumClass"}, new String[]{TaskState.class.getName()});

  public HibernateXpdlPersistenceService(Session session) {
    setSession(session);
    session.setFlushMode(FlushMode.COMMIT);
  }

  public TaskRunTime getTask(TaskUUID taskUUID) {
    Query query = getSession().getNamedQuery("getTask");
    query.setString("taskUUID", taskUUID.toString());
    query.setMaxResults(1);
    query.setCacheable(true);
    TaskRunTime task = (TaskRunTime) query.uniqueResult();
    return task;
  }

  @SuppressWarnings("unchecked")
  public List<TaskRunTime> getInstanceTasks(ProcessInstanceUUID instanceUUID) {
    List<TaskRunTime> result = new ArrayList<TaskRunTime>();
    Query query = getSession().getNamedQuery("getInstanceTasks");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    result.addAll(query.list());
    return result;
  }

  @SuppressWarnings("unchecked")
  public Set<ActivityFullInstance<TaskInstance>> getUserInstanceTasks(String userId,
      ProcessInstanceUUID instanceUUID, TaskState taskState) {
    Set<ActivityFullInstance<TaskInstance>> result = new HashSet<ActivityFullInstance<TaskInstance>>();
    Query query = getSession().getNamedQuery("getUserInstanceTasksWithState");
    query.setCacheable(true);
    query.setString("userId", userId);
    query.setString("instanceUUID", instanceUUID.toString());
    query.setParameter("state", taskState, taskStateUserType);
    result.addAll(query.list());
    return result;
  }
  @SuppressWarnings("unchecked")
  public Set<ActivityFullInstance<TaskInstance>> getUserTasks(String userId, TaskState taskState) {
    Query query = getSession().getNamedQuery("getUserTasksWithState");
    query.setCacheable(true);
    query.setString("userId", userId);
    query.setParameter("state", taskState, taskStateUserType);
    Set<ActivityFullInstance<TaskInstance>> result = new HashSet<ActivityFullInstance<TaskInstance>>();
    result.addAll(query.list());
    return result;
  }

  public XpdlInstance getXpdlInstance(ProcessInstanceUUID instanceUUID) {
    Query query = getSession().getNamedQuery("getInstanceByUUID");
    query.setString("instanceUUID", instanceUUID.toString());
    query.setCacheable(true);
    query.setMaxResults(1);
    return (XpdlInstance) query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public List<XpdlInstance> getXpdlInstances(ProcessDefinitionUUID processId) {
    Query query = getSession().getNamedQuery("getInstances");
    query.setCacheable(true);
    query.setString("processUUID", processId.toString());
    return query.list();
  }

  public XpdlProcess findXpdlProcessById(ProcessDefinitionUUID processUUID) {
    Query query = getSession().getNamedQuery("findLatestXpdlProcessByUUID");
    query.setString("processUUID", processUUID.toString());
    query.setCacheable(true);
    query.setMaxResults(1);
    XpdlProcess processDefinition = (XpdlProcess) query.uniqueResult();
    return processDefinition;
  }

  @SuppressWarnings("unchecked")
  public List<XpdlProcess> getXpdlProcesses() {
    Query query = getSession().getNamedQuery("getAllProcesses");
    query.setCacheable(true);
    return query.list();
  }

  public boolean hasProcessWithUUID(ProcessDefinitionUUID processUUID) {
    Query query = getSession().getNamedQuery("hasProcessWithUUID");
    query.setCacheable(true);
    query.setString("processUUID", processUUID.toString());
    ProcessFullDefinition returnedProcess = (ProcessFullDefinition) query.uniqueResult();
    boolean hasProcess = returnedProcess != null;
    return hasProcess;
  }

  public boolean hasPackageWithUUID(PackageDefinitionUUID packageUUID) {
    Query query = getSession().getNamedQuery("hasPackageWithUUID");
    query.setCacheable(true);
    query.setString("packageUUID", packageUUID.toString());
    PackageFullDefinition packageDef = (PackageFullDefinition) query.uniqueResult();
    boolean hasProcess = packageDef != null;
    return hasProcess;
  }

  public PackageClassData getPackageClassData(PackageDefinitionUUID packageUUID) {
    Query query = getSession().getNamedQuery("findClassDataForPackageWithUUID");
    query.setCacheable(true);
    query.setString("packageUUID", packageUUID.toString());
    query.setMaxResults(1);
    PackageClassData classData = (PackageClassData) query.uniqueResult();
    return classData;
  }

  public GlobalClassData getGlobalClassData(String className) {
    Query query = getSession().getNamedQuery("findGlobalClassData");
    query.setCacheable(true);
    query.setString("className", className);
    query.setMaxResults(1);
    GlobalClassData classData = (GlobalClassData) query.uniqueResult();
    return classData;
  }

  @SuppressWarnings("unchecked")
  public ActivityFullInstance<ActivityBody> getActivityRecord(
      ProcessInstanceUUID instanceUUID, String activityId, String iterationId) {
    Query query = getSession().getNamedQuery("findActivityInstance");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    query.setString("activityId", activityId);
    query.setString("iterationId", iterationId);
    query.setMaxResults(1);
    return (ActivityFullInstance<ActivityBody>) query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public Set<ActivityFullInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID) {
    Query query = getSession().getNamedQuery("findActivityInstances");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    List<ActivityFullInstance<ActivityBody>> results = query.list();
    if (results != null) {
      return new HashSet<ActivityFullInstance<ActivityBody>>(results);
    }
    return null;
  }

  public ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID) {
    Query query = getSession().getNamedQuery("findProcessInstance");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    query.setMaxResults(1);
    return (ProcessFullInstance) query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public Set<ProcessFullInstance> getProcessInstances(final ProcessDefinitionUUID processUUID) {
    final Set<ProcessFullInstance> processInsts = new HashSet<ProcessFullInstance>();
    final Query query = getSession().getNamedQuery("findProcessInstances");
    query.setCacheable(true);
    query.setString("processUUID", processUUID.toString());
    final List<ProcessFullInstance> results = query.list();
    if (results != null) {
      processInsts.addAll(results);
    }
    return processInsts;
  }

  @SuppressWarnings("unchecked")
  public ActivityFullInstance< TaskInstance > getTaskInstance(TaskUUID taskUUID) {
    Query query = getSession().getNamedQuery("findTaskInstance");
    query.setCacheable(true);
    query.setString("taskUUID", taskUUID.toString());
    query.setMaxResults(1);
    return (ActivityFullInstance< TaskInstance >) query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public Set<ActivityFullInstance< TaskInstance >> getTaskInstances(final ProcessInstanceUUID instanceUUID) {
    final Query query = getSession().getNamedQuery("findTaskInstances");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    final Set<ActivityFullInstance< TaskInstance >> taskInstances = new HashSet<ActivityFullInstance< TaskInstance >>();
    final List<ActivityFullInstance<TaskInstance>> results = query.list();
    if (results != null) {
      taskInstances.addAll(results);
    }
    return taskInstances;
  }

  public XpdlExecution getExecutionPointingOnNode(ProcessInstanceUUID instanceUUID, String nodeName) {
    Query query = getSession().getNamedQuery("findInstanceExecutionPointingOnNode");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    query.setString("nodeName", nodeName);
    query.setMaxResults(1);
    return (XpdlExecution) query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public Set<PackageDependency> getPackageDependencies(String className) {
    final Query query = getSession().getNamedQuery("getPackageDepencies");
    query.setCacheable(true);
    query.setString("className", className);
    final List<PackageDependency> results = query.list();
    if (results != null) {
      return new HashSet<PackageDependency>(results);
    }
    return null;
  }

  public PackageDependency getPackageDependency(PackageDefinitionUUID packageUUID) {
    final Query query = getSession().getNamedQuery("getPackageDependency");
    query.setCacheable(true);
    query.setString("packageUUID", packageUUID.toString());
    return (PackageDependency) query.uniqueResult();
  }

  /*
   * DEFINITION
   */
  @SuppressWarnings("unchecked")
  private Set<PackageFullDefinition> getPackageSet(final Query query) {
    final List<PackageFullDefinition> results = query.list();
    if (results != null) {
      return new HashSet<PackageFullDefinition>(results);
    }
    return null;
  }


  @SuppressWarnings("unchecked")
  private Set<ProcessFullDefinition> getProcessSet(final Query query) {
    final List<ProcessFullDefinition> results = query.list();
    if (results != null) {
      return new HashSet<ProcessFullDefinition>(results);
    }
    return null;
  }

  @SuppressWarnings("unchecked")
  private Set<String> getStringSet(final Query query) {
    final List<String> results = query.list();
    if (results != null) {
      return new HashSet<String>(results);
    }
    return null;
  }


  public Set<PackageFullDefinition> getPackages() {
    final Query query = getSession().getNamedQuery("getPackages1");
    query.setCacheable(true);
    return getPackageSet(query);
  }

  public Set<PackageFullDefinition> getPackages(String packageId) {
    final Query query = getSession().getNamedQuery("getPackages2");
    query.setCacheable(true);
    query.setString("packageId", packageId);
    return getPackageSet(query);
  }

  public PackageFullDefinition getPackage(PackageDefinitionUUID packageUUID) {
    final Query query = getSession().getNamedQuery("getPackage");
    query.setCacheable(true);
    query.setString("packageUUID", packageUUID.toString());
    query.setMaxResults(1);
    return (PackageFullDefinition) query.uniqueResult();
  }

  public Set<PackageFullDefinition> getPackages(PackageDefinition.PackageState packageState) {
    Query query = null;
    if (PackageDefinition.PackageState.DEPLOYED.equals(packageState)) {
      query = getSession().getNamedQuery("getDeployedPackages1");
    } else if (PackageDefinition.PackageState.UNDEPLOYED.equals(packageState)) {
      query = getSession().getNamedQuery("getUndeployedPackages1");
    } else {
      return null;
    }
    query.setCacheable(true);
    return getPackageSet(query);
  }

  public Set<PackageFullDefinition> getPackages(String packageId, PackageDefinition.PackageState packageState) {
    Query query = null;
    if (PackageDefinition.PackageState.DEPLOYED.equals(packageState)) {
      query = getSession().getNamedQuery("getDeployedPackages2");
    } else if (PackageDefinition.PackageState.UNDEPLOYED.equals(packageState)) {
      query = getSession().getNamedQuery("getUndeployedPackages2");
    } else {
      return null;
    }
    query.setCacheable(true);
    query.setString("packageId", packageId);
    return getPackageSet(query);
  }

  public PackageFullDefinition getLastDeployedPackage(String packageId) {
    final Query query = getSession().getNamedQuery("getLastDeployedPackage");
    query.setCacheable(true);
    query.setString("packageId", packageId);
    query.setMaxResults(1);
    return (PackageFullDefinition) query.uniqueResult();
  }

  public Set<ProcessFullDefinition> getPackageProcesses(PackageDefinitionUUID packageId) {
    final Query query = getSession().getNamedQuery("getPackageProcesses");
    query.setCacheable(true);
    query.setString("packageUUID", packageId.toString());
    return getProcessSet(query);
  }

  public Set<ProcessFullDefinition> getProcesses() {
    final Query query = getSession().getNamedQuery("getProcesses1");
    query.setCacheable(true);
    return getProcessSet(query);
  }

  public Set<ProcessFullDefinition> getProcesses(String processId) {
    final Query query = getSession().getNamedQuery("getProcesses2");
    query.setCacheable(true);
    query.setString("processId", processId);
    return getProcessSet(query);
  }

  public ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID) {
    final Query query = getSession().getNamedQuery("getProcess");
    query.setCacheable(true);
    query.setString("processUUID", processUUID.toString());
    query.setMaxResults(1);
    return (ProcessFullDefinition) query.uniqueResult();
  }

  public Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState) {
    Query query = null;
    if (ProcessDefinition.ProcessState.DEPLOYED.equals(processState)) {
      query = getSession().getNamedQuery("getDeployedProcesses1");
    } else if (ProcessDefinition.ProcessState.UNDEPLOYED.equals(processState)) {
      query = getSession().getNamedQuery("getUndeployedProcesses1");
    } else {
      return null;
    }
    query.setCacheable(true);
    return getProcessSet(query);
  }

  public Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState) {
    Query query = null;
    if (ProcessDefinition.ProcessState.DEPLOYED.equals(processState)) {
      query = getSession().getNamedQuery("getDeployedProcesses2");
    } else if (ProcessDefinition.ProcessState.UNDEPLOYED.equals(processState)) {
      query = getSession().getNamedQuery("getUndeployedProcesses2");
    } else {
      return null;
    }
    query.setCacheable(true);
    query.setString("processId", processId);
    return getProcessSet(query);
  }

  public ProcessFullDefinition getLastDeployedProcess(String processId) {
    final Query query = getSession().getNamedQuery("getLastDeployedProcess");
    query.setCacheable(true);
    query.setString("processId", processId);
    query.setMaxResults(1);
    return (ProcessFullDefinition) query.uniqueResult();
  }

  public Set<String> getDependentPackages(PackageDefinitionUUID packageUUID) {
    Query query = getSession().getNamedQuery("getDependentPackages");
    query.setCacheable(true);
    query.setString("packageUUID", packageUUID.toString());
    return getStringSet(query);
  }

  @SuppressWarnings("unchecked")
  public List<Timer> getInstanceTimers(ProcessInstanceUUID instanceUUID) {
    Query query = getSession().getNamedQuery("getInstanceTimers");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    return query.list();
  }

  @SuppressWarnings("unchecked")
  public ActivityFullInstance<ActivityBody> getActivityInstance(
      ProcessInstanceUUID instanceUUID, String activityId, String iterationId, String activityInstanceId) {
    Query query = getSession().getNamedQuery("getActivityInstance");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    query.setString("activityId", activityId);
    query.setString("iterationId", iterationId);
    query.setString("activityInstanceId", activityInstanceId);
    return (ActivityFullInstance<ActivityBody>)query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public ActivityFullInstance<ActivityBody> getActivityInstance(
      ActivityInstanceUUID activityInstanceUUID) {
    Query query = getSession().getNamedQuery("getActivityInstanceFromUUID");
    query.setCacheable(true);
    query.setString("activityUUID", activityInstanceUUID.toString());
    return (ActivityFullInstance<ActivityBody>)query.uniqueResult();
  }

  public ProcessFullDefinition getPackageProcess(
      PackageDefinitionUUID packageDefinitionUUID, String processId) {
    Query query = getSession().getNamedQuery("getPackageProcess");
    query.setCacheable(true);
    query.setString("packageUUID", packageDefinitionUUID.toString());
    query.setString("processId", processId);
    return (ProcessFullDefinition)query.uniqueResult();
  }

  public XpdlProcess findLatestProcessById(String processId) {
    Query query = getSession().getNamedQuery("findLatestProcessById");
    query.setCacheable(true);
    query.setString("processId", processId);
    return (XpdlProcess)query.uniqueResult();
  }

  public ActivityInstanceKey getActivityInstanceKey(ProcessInstanceUUID instanceUUID,
      String activityName) {
    Query query = getSession().getNamedQuery("getActivityInstanceKey");
    query.setCacheable(true);
    query.setString("instanceUUID", instanceUUID.toString());
    query.setString("activityId", activityName);
    return (ActivityInstanceKey)query.uniqueResult();
  }

  public ProcessInstanceKey getProcessInstanceKey(ProcessDefinitionUUID processUUID) {
    Query query = getSession().getNamedQuery("getProcessInstanceKey");
    query.setCacheable(true);
    query.setString("processUUID", processUUID.toString());
    return (ProcessInstanceKey)query.uniqueResult();
  }

  @SuppressWarnings("unchecked")
  public Collection<Timer> getExecutionTimers(XpdlExecution xpdlExecution) {
    Query query = getSession().createQuery("from " + TimerImpl.class.getName() + " as timer where timer.execution = :exec");
    query.setEntity("exec", xpdlExecution);
    return query.list();
  }


  /**
   * TEMPORARY FIX FOR TIMERS THAT ARE DELETED TWICE.
   * TODO: change timers in the pvm and remove this method.
   */
  @Override
  public void delete(Object entity) {
    if (getSession().contains(entity)) {
      super.delete(entity);
    } else {
      LOG.warning("Trying to delete an entity that is not managed by hibernate: " + entity);
    }
  }

  public ActivityDefinition getActivityDefinition(ActivityDefinitionUUID activityDefinitionUUID) {
    final Query query = getSession().getNamedQuery("getActivityDefinition");
    query.setCacheable(true);
    query.setString("activityUUID", activityDefinitionUUID.toString());
    query.setMaxResults(1);
    return (ActivityDefinition) query.uniqueResult();
  }
}
