package org.ow2.bonita.runtime;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.jbpm.pvm.job.Timer;
import org.ow2.bonita.definition.GlobalClassData;
import org.ow2.bonita.definition.PackageClassData;
import org.ow2.bonita.definition.PackageDependency;
import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;

/**
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public interface Repository {

  /**
   * The key used by the IoC engine for concrete implementation injection.
   */
  String DEFAULT_KEY = "repository";

  /**
   * CLASS
   */
  void storePackageClassData(PackageClassData packageClassData);
  void removePackageClassData(PackageDefinitionUUID packageUUID);
  //if found in package datas take it, else look into global datas
  PackageClassData getPackageClassData(PackageDefinitionUUID packageId);
  void storeGlobalClassData(GlobalClassData globalClassData);
  void removeGlobalClassData(String className);
  GlobalClassData getGlobalClassData(String className);

  /**
   * INSTANCE
   */
  void storeInstance(XpdlInstance instance);
  Set<XpdlInstance> getXpdlInstances(ProcessDefinitionUUID processId);
  XpdlInstance getXpdlInstance(ProcessInstanceUUID instanceUUID);
  XpdlInstance removeXpdlInstance(ProcessInstanceUUID instanceUUID);
  XpdlExecution getExecutionPointingOnNode(ProcessInstanceUUID instanceUUID, String nodeName);

  /**
   * PROCESS
   */
  void storeProcess(XpdlProcess xpdlProcess);

  List<XpdlProcess> getXpdlProcesses();
  XpdlProcess findXpdlProcessById(ProcessDefinitionUUID processUUID);
  XpdlProcess removeProcess(ProcessDefinitionUUID processUUID);
  boolean hasProcessWithUUID(ProcessDefinitionUUID processUUID);
  boolean hasPackageWithUUID(PackageDefinitionUUID packageUUID);

  /**
   * TASK
   */
  /** return all tasks associated to the given instance */
  List<TaskRunTime> getInstanceTasks(ProcessInstanceUUID instanceUUID);
  TaskRunTime getTask(TaskUUID taskUUID);
  
  void addTask(TaskRunTime task);
  void removeTask(TaskRunTime task);
  

  /*
   * CLASS DEPENDENCIES
   */
  void addPackageDependencies(PackageDefinitionUUID packageUUID, Set<String> classNames);
  void removePackageDependencies(PackageDefinitionUUID packageUUID);
  PackageDependency getPackageDependency(PackageDefinitionUUID packageUUID);
  /** returns a set of packageDefinitionUUID of all packages depending on the given className. */
  Set<PackageDependency> getPackageDependencies(String className);

  /*
   * TIMERS
   */
  void addTimer(Timer timer);
  /** return all timers associated to the given instance */
  List<Timer> getInstanceTimers(ProcessInstanceUUID instanceUUID);
  /** return all timers associated to the given execution */
  Collection<Timer> getExecutionTimers(XpdlExecution xpdlExecution);
  void removeTimer(Timer timer);
  XpdlProcess findLatestProcessById(String subProcessId);
  long getNextActivityInstanceNb(ProcessInstanceUUID instanceUUID,
      String activityName);
  long getNextProcessInstanceNb(ProcessDefinitionUUID processUUID);
}
