package org.ow2.bonita.runtime;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.ow2.bonita.definition.XpdlProcess;
import org.ow2.bonita.facade.runtime.InstanceState;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.util.BonitaRuntimeException;
import org.ow2.bonita.util.EnvTool;


public class XpdlInstance {

  protected InstanceState instanceState;
  //processInstanceLevel
  protected ProcessInstanceUUID uuid;
  protected long dbid;
  protected ProcessDefinitionUUID processUUID;
  protected PackageDefinitionUUID packageUUID;
  protected XpdlExecution rootExecution;
  /**
   * List of runtime execution scopes added for runtime informations other than
   * variables (protected List<VariableScope> variableScopes;)
   */
  protected List<JoinRunTime> joinRuntimes;
  protected XpdlExecution executionToSignal;
  protected Set<XpdlInstance> childInstances;
  protected XpdlInstance parentInstance;

  protected XpdlInstance() { }

  public XpdlInstance(XpdlProcess xpdlProcess) {
    this.uuid = new ProcessInstanceUUID(xpdlProcess.getProcessDefinitionUUID());
    this.processUUID = xpdlProcess.getProcessDefinitionUUID();
    this.packageUUID = xpdlProcess.getPackageDefinitionUUID();
    this.rootExecution = new XpdlExecution();
    this.rootExecution.xpdlInstance = this;
    // TODO: process is set in startExecution too, see how we can remove this.
    // (process is needed to assign instance name from the instance repository)
    this.rootExecution.setProcessDefinition(xpdlProcess);

    final Repository repository = EnvTool.getRepository();
    repository.storeInstance(this);
    this.rootExecution.setName(this.uuid.toString());
  }

  public void setInstanceState(final InstanceState state) {
    this.instanceState = state;
  }

  public boolean isInstanceState(final InstanceState state) {
    return this.instanceState.equals(state);
  }

  public void  pushJoinRuntime(JoinRunTime runTime) {
    if (getJoinRuntimes() == null) {
      joinRuntimes = new ArrayList<JoinRunTime>();
    }
    getJoinRuntimes().add(0, runTime);
  }

  public void popJoinRuntime() {
    if ((getJoinRuntimes() != null) && (!getJoinRuntimes().isEmpty())) {
      getJoinRuntimes().remove(0);
    }
  }
  public List<JoinRunTime> getJoinRuntimes() {
    return joinRuntimes;
  }
  public JoinRunTime getJoinRunTime(String nodeName) {
    JoinRunTime joinRunTime = null;
    if (joinRuntimes != null) {
      for (JoinRunTime jr : joinRuntimes) {
        if (jr.getNodeName().equals(nodeName)) {
          if (joinRunTime != null) {
            throw new BonitaRuntimeException("More than 1 joinRuntime was found for the given nodeName : " + nodeName);          
          }
          joinRunTime = jr;
        }
      }
    }
    return joinRunTime;
  }

  public XpdlExecution getRootExecution() {
    return rootExecution;
  }

  public InstanceState getInstanceState() {
    return instanceState;
  }

  public ProcessInstanceUUID getUUID() {
    return uuid;
  }


  public long getDbid() {
    return dbid;
  }


  public ProcessDefinitionUUID getProcessDefinitionUUID() {
    return processUUID;
  }

  /**
   * @param execution
   */
  public void setExecutionToSignal(XpdlExecution execution) {
    this.executionToSignal = execution;
    this.parentInstance = execution.getXpdlInstance();
    this.parentInstance.addChildInstance(this);
  }

  public XpdlExecution getExecutionToSignal() {
    return executionToSignal;
  }

  private void addChildInstance(XpdlInstance instance) {
    if (childInstances == null) {
      childInstances = new HashSet<XpdlInstance>();
    }
    childInstances.add(instance);
  }

  @Override
  public String toString() {
    String value = "Instance " + uuid + "(state:" + instanceState;
    if (parentInstance != null) {
      value += ", child of " + parentInstance;
    }
    value += ")";
    return value;
  }


  public Set<XpdlInstance> getChildInstances() {
    if (childInstances == null) {
      return null;
    }
    return Collections.unmodifiableSet(childInstances);
  }

  public XpdlInstance getParentInstance() {
    return parentInstance;
  }

  public PackageDefinitionUUID getPackageDefinitionUUID() {
    return packageUUID;
  }

}
