/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services;

import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.runtime.TaskState;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.ActivityInstanceUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.facade.uuid.TaskUUID;



/**
 * @author Pierre Vigneras
 * @date Jan 30, 2008
 */
public interface Querier {

  String DEFAULT_KEY = "queryList";

  /*
   * ACTIVITY
   */
  Set<ActivityFullInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID);
  ActivityFullInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID, 
      String activityId, String iterationId, String activityInstanceId);
  ActivityFullInstance<ActivityBody> getActivityInstance(ActivityInstanceUUID activityInstanceUUID);

  /*
   * HOOK
   */
  //Map<JavaHook.Type, HookInstance> getHooks(String processUUID, String activityId);

  /*
   * INSTANCE
   */
  Set<ProcessFullInstance> getProcessInstances(ProcessDefinitionUUID processUUID);
  ProcessFullInstance getProcessInstance(ProcessInstanceUUID instanceUUID);

  /*
   * TASK
   */
  Set<ActivityFullInstance<TaskInstance>> getTaskInstances(ProcessInstanceUUID instanceUUID);
  ActivityFullInstance<TaskInstance> getTaskInstance(TaskUUID taskUUID);
  /** return all tasks associated to the given user and having the given state */
  Set<ActivityFullInstance<TaskInstance>> getUserTasks(String userId, TaskState taskState);
  /** return all tasks associated to the given user and the given instance and having the given state */
  Set<ActivityFullInstance<TaskInstance>> getUserInstanceTasks(String userId, 
      ProcessInstanceUUID instanceUUID, TaskState taskState);
  
  /*
   * DEFINITION
   */
  Set<PackageFullDefinition> getPackages();
  Set<PackageFullDefinition> getPackages(String packageId);
  PackageFullDefinition getPackage(PackageDefinitionUUID packageUUID);
  Set<PackageFullDefinition> getPackages(PackageDefinition.PackageState packageState);
  Set<PackageFullDefinition> getPackages(String packageId, PackageDefinition.PackageState packageState);
  PackageFullDefinition getLastDeployedPackage(String packageId);
  ProcessFullDefinition getPackageProcess(PackageDefinitionUUID packageDefinitionUUID, String processId);

  Set<ProcessFullDefinition> getPackageProcesses(PackageDefinitionUUID packageUUID);
  Set<ProcessFullDefinition> getProcesses();
  Set<ProcessFullDefinition> getProcesses(String processId);
  Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState);
  Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState);
  ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID);
  ProcessFullDefinition getLastDeployedProcess(String processId);

  Set<String> getDependentPackages(PackageDefinitionUUID packageUUID);

  ActivityDefinition getActivity(ActivityDefinitionUUID activityDefinitionUUID);

}
