/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.services.impl;

import java.util.HashSet;
import java.util.Set;

import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageDefinition;
import org.ow2.bonita.facade.def.majorElement.PackageFullDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.def.majorElement.ProcessFullDefinition;
import org.ow2.bonita.facade.runtime.ActivityBody;
import org.ow2.bonita.facade.runtime.ActivityFullInstance;
import org.ow2.bonita.facade.runtime.ProcessFullInstance;
import org.ow2.bonita.facade.runtime.TaskInstance;
import org.ow2.bonita.facade.uuid.ActivityDefinitionUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessInstanceUUID;
import org.ow2.bonita.services.History;
import org.ow2.novabpm.util.Misc;


/**
 * @author Pierre Vigneras
 * @date Mar 7, 2008
 */
public abstract class AbstractHistory implements History {

  /*public ActivityFullInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID,
      String activityId, String iterationId) {
    
    Misc.checkArgsNotNull(instanceUUID, activityId, iterationId);
    Set<ActivityFullInstance<ActivityBody>> activities = getActivityInstances(instanceUUID);
    if (activities != null) {
      for (ActivityFullInstance<ActivityBody> activity : activities) {
        if (activity.getActivityId().equals(activityId) && activity.getIterationId().equals(iterationId)) {
          return activity;
        }
      }
    }
    return null;
  }*/
  public ActivityFullInstance<ActivityBody> getActivityInstance(ProcessInstanceUUID instanceUUID,
      String activityId, String iterationId, String activityInstanceId) {
    Misc.checkArgsNotNull(instanceUUID, activityId, iterationId, activityInstanceId);
    ProcessFullInstance instance = getProcessInstance(instanceUUID);
    if (instance == null) {
      return null;
    }
    return instance.getActivity(activityId, iterationId, activityInstanceId);
  }
  
  public Set<ActivityFullInstance<ActivityBody>> getActivityInstances(ProcessInstanceUUID instanceUUID) {
    Misc.checkArgsNotNull(instanceUUID);
    ProcessFullInstance instance = getProcessInstance(instanceUUID);
    if (instance == null) {
      return new HashSet<ActivityFullInstance<ActivityBody>>();
    }
    Set<ActivityFullInstance<ActivityBody>> result = instance.getActivities();
    if (result == null) {
      return new HashSet<ActivityFullInstance<ActivityBody>>();
    }
    return result;
  }
  
  @SuppressWarnings("unchecked")
  public Set<ActivityFullInstance< TaskInstance >> getTaskInstances(ProcessInstanceUUID instanceUUID) {
    Misc.checkArgsNotNull(instanceUUID);
    final Set<ActivityFullInstance< TaskInstance >> result = new HashSet<ActivityFullInstance< TaskInstance >>();

    Set<ActivityFullInstance<ActivityBody>> activities = getActivityInstances(instanceUUID);
    if (activities != null) {
      for (ActivityFullInstance< ? > activity : activities) {
        ActivityBody body = activity.getBody();
        if (body != null && body instanceof TaskInstance) {
          result.add((ActivityFullInstance<TaskInstance>) activity);
        }
      }
    }
    return result;
  }
  
  /*
  public Set<PackageFullDefinition> getLastDeployedPackages() {
    final Map<String, PackageFullDefinition>  result = new HashMap<String, PackageFullDefinition>();
    final Set<PackageFullDefinition>  all = getPackages();
    for (PackageFullDefinition candidate : all) {
      PackageFullDefinition oldCandidate = result.get(candidate.getId());
      if (oldCandidate != null) {
        if (oldCandidate.getDeployedDate().after(candidate.getDeployedDate())) {
          candidate = oldCandidate;
        }
      }
      result.put(candidate.getId(), candidate);
    }
    return new HashSet<PackageFullDefinition>(result.values());
  }

  public PackageFullDefinition getLastDeployedPackage(String packageDefinitionUUID) {
    PackageFullDefinition result = null;
    final Set<PackageFullDefinition>  all = getPackages();
    for (PackageFullDefinition candidate : all) {
      if (candidate.getId().equals(packageDefinitionUUID)) {
        if (result == null || candidate.getDeployedDate().after(result.getDeployedDate())) {
          result = candidate;
        }
      }
    }
    return result;
  }

  public Set<PackageFullDefinition> getPackages(String packageDefinitionUUID) {
    final Set<PackageFullDefinition>  result = new HashSet<PackageFullDefinition>();
    final Set<PackageFullDefinition>  all = getPackages();
    for (PackageFullDefinition record : all) {
      if (record.getId().equals(packageDefinitionUUID)) {
        result.add(record);
      }
    }
    return result;
  }

  public PackageFullDefinition getPackage(String packageDefinitionUUID, String deploymentId) {
    for (PackageFullDefinition record : getPackages()) {
      if (record.getId().equals(packageDefinitionUUID) && record.getDeploymentId().equals(deploymentId)) {
        return record;
      }
    }
    return null;
  }*/
  
  public Set<PackageFullDefinition> getPackages(String packageId) {
    Misc.checkArgsNotNull(packageId);
    Set<PackageFullDefinition> packages = getPackages();
    Set<PackageFullDefinition> result = new HashSet<PackageFullDefinition>();
    if (packages != null) {
      for (PackageFullDefinition package1 : packages) {
        if (package1.getPackageId().equals(packageId)) {
          result.add(package1);
        }
      }
    }
    return result;
  }

  public ProcessFullDefinition getPackageProcess(
      PackageDefinitionUUID packageDefinitionUUID, String processId) {

    Misc.checkArgsNotNull(packageDefinitionUUID, processId);
    PackageFullDefinition packageDef = getPackage(packageDefinitionUUID);
    if (packageDef != null) {
      Set<ProcessFullDefinition> processes = packageDef.getProcesses();
      for (ProcessFullDefinition process : processes) {
        if (process.getProcessId().equals(processId)) {
          return process;
        }
      }
    }
    return null;
  }

  
  public Set<PackageFullDefinition> getPackages(PackageDefinition.PackageState packageState) {
    Misc.checkArgsNotNull(packageState);
    Set<PackageFullDefinition> packages = getPackages();
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    Set<PackageFullDefinition> result = new HashSet<PackageFullDefinition>();
    for (PackageFullDefinition package1 : packages) {
      if (PackageDefinition.PackageState.DEPLOYED.equals(packageState) && package1.getUndeployedBy() == null) {
        result.add(package1);
      } else if (PackageDefinition.PackageState.UNDEPLOYED.equals(packageState)) {
        result.add(package1);
      }
    }
    return result;
  }

  public Set<PackageFullDefinition> getPackages(String packageId, PackageDefinition.PackageState packageState) {
    Set<PackageFullDefinition> packages = getPackages(packageId);
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    Set<PackageFullDefinition> result = new HashSet<PackageFullDefinition>();
    for (PackageFullDefinition package1 : packages) {
      if (PackageDefinition.PackageState.DEPLOYED.equals(packageState) && package1.getUndeployedBy() == null) {
        result.add(package1);
      } else if (PackageDefinition.PackageState.UNDEPLOYED.equals(packageState)) {
        result.add(package1);
      }
    }
    return result;
  }

  public PackageFullDefinition getLastDeployedPackage(String packageId) {
    Set<PackageFullDefinition> packages = getPackages(packageId);
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    PackageFullDefinition last = null;
    for (PackageFullDefinition package1 : packages) {
      if (last == null || package1.getDeployedDate().after(last.getDeployedDate())) {
        last = package1;
      }
    }
    return last;
  }

  public Set<ProcessFullDefinition> getPackageProcesses(PackageDefinitionUUID packageId) {
    PackageFullDefinition package1 = getPackage(packageId);
    if (package1 == null) {
      return null;
    }
    return package1.getProcesses();
  }

  public Set<ProcessFullDefinition> getProcesses() {
    Set<PackageFullDefinition> packages = getPackages();
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    Set<ProcessFullDefinition> result = new HashSet<ProcessFullDefinition>();
    for (PackageFullDefinition package1 : packages) {
      Set<ProcessFullDefinition> processes = package1.getProcesses();
      if (processes != null && !processes.isEmpty()) {
        result.addAll(processes);
      }
    }
    return result;
  }

  public Set<ProcessFullDefinition> getProcesses(String processId) {
    Set<PackageFullDefinition> packages = getPackages();
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    Set<ProcessFullDefinition> result = new HashSet<ProcessFullDefinition>();
    for (PackageFullDefinition package1 : packages) {
      Set<ProcessFullDefinition> processes = package1.getProcesses();
      if (processes != null && !processes.isEmpty()) {
        for (ProcessFullDefinition process : processes) {
          if (process.getProcessId().equals(processId)) {
            result.add(process);
          }
        }
      }
    }
    return result;
  }

  public Set<ProcessFullDefinition> getProcesses(ProcessDefinition.ProcessState processState) {
    Set<PackageFullDefinition> packages = getPackages();
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    Set<ProcessFullDefinition> result = new HashSet<ProcessFullDefinition>();
    for (PackageFullDefinition package1 : packages) {
      boolean match = false;
      if (ProcessDefinition.ProcessState.DEPLOYED.equals(processState) && package1.getUndeployedBy() == null) {
        match = true;
      } else if (ProcessDefinition.ProcessState.UNDEPLOYED.equals(processState)) {
        match = true;
      }
      if (match) {
        Set<ProcessFullDefinition> processes = package1.getProcesses();
        if (processes != null && !processes.isEmpty()) {
          result.addAll(processes);
        }
      }
    }
    return result;
  }

  public Set<ProcessFullDefinition> getProcesses(String processId, ProcessDefinition.ProcessState processState) {
    Set<PackageFullDefinition> packages = getPackages();
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    Set<ProcessFullDefinition> result = new HashSet<ProcessFullDefinition>();
    for (PackageFullDefinition package1 : packages) {
      boolean match = false;
      if (ProcessDefinition.ProcessState.DEPLOYED.equals(processState) && package1.getUndeployedBy() == null) {
        match = true;
      } else if (ProcessDefinition.ProcessState.UNDEPLOYED.equals(processState)) {
        match = true;
      }
      if (match) {
        Set<ProcessFullDefinition> processes = package1.getProcesses();
        for (ProcessFullDefinition process : processes) {
          if (process.getProcessId().equals(processId)) {
            result.add(process);
          }
        }
      }
    }
    return result;
  }

  public ProcessFullDefinition getProcess(ProcessDefinitionUUID processUUID) {
    Set<PackageFullDefinition> packages = getPackages();
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    for (PackageFullDefinition package1 : packages) {
      Set<ProcessFullDefinition> processes = package1.getProcesses();
      if (processes != null && !processes.isEmpty()) {
        for (ProcessFullDefinition process : processes) {
          if (process.getUUID().equals(processUUID)) {
            return process;
          }
        }
      }
    }
    return null;
  }

  public ProcessFullDefinition getLastDeployedProcess(String processId) {
    Set<PackageFullDefinition> packages = getPackages();
    if (packages == null || packages.isEmpty()) {
      return null;
    }
    PackageFullDefinition lastPackage = null;
    ProcessFullDefinition lastProcess = null;
    for (PackageFullDefinition package1 : packages) {
      if (lastPackage == null) {
        lastPackage = package1;
      }
      Set<ProcessFullDefinition> processes = package1.getProcesses();
      if (processes != null && !processes.isEmpty()) {
        for (ProcessFullDefinition process : processes) {
          if (process.getProcessId().equals(processId)) {
            if (lastProcess == null || package1.getDeployedDate().after(lastPackage.getDeployedDate())) {
              lastPackage = package1;
              lastProcess = process;
            }
          }
        }
      }
    }
    return lastProcess;
  }

  public Set<String> getDependentPackages(PackageDefinitionUUID packageId) {
    Set<String> result = new HashSet<String>();
    Set<PackageFullDefinition> deployedPackages = getPackages(PackageDefinition.PackageState.DEPLOYED);
    for (PackageFullDefinition package1 : deployedPackages) {
      if (!package1.getPackageDefinitionUUID().equals(packageId)) {
        if (package1.getPackageDependencies() != null && package1.getPackageDependencies().contains(packageId)) {
          result.add(package1.getPackageId());
        }
      }
    }
    return result;
  }
  
  public ActivityDefinition getActivity(ActivityDefinitionUUID activityDefinitionUUID) {
    Set<ProcessFullDefinition> processes = getProcesses();
    if (processes != null) {
      for (ProcessFullDefinition process : processes) {
        Set<ActivityDefinition> activities = process.getActivities();
        if (activities != null) {
          for (ActivityDefinition activity : activities) {
            if (activity.getUUID().equals(activityDefinitionUUID)) {
              return activity;
            }
          }
        }
      }
    }
    return null;
  }
}
