package org.ow2.bonita.textui;

import java.io.File;
import java.io.IOException;

import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;

import org.ow2.bonita.facade.ManagementAPI;
import org.ow2.bonita.facade.QueryDefinitionAPI;
import org.ow2.bonita.facade.def.majorElement.ProcessDefinition;
import org.ow2.bonita.facade.exception.ProcessNotFoundException;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.util.AccessorUtil;
import org.ow2.bonita.util.BonitaException;
import org.ow2.novabpm.util.Misc;
import org.ow2.novabpm.util.SimpleCallbackHandler;
import org.ow2.novabpm.util.StandardCallbackHandler;


public class Console {

  public static final String DEPLOY = "deploy";
  public static final String UNDEPLOY = "undeploy";
  public static final String ACTION_USAGE = "Action must be '" + DEPLOY + "' or '" + UNDEPLOY + "' ";
  public static final String USAGE = "Usage: " + Console.class + DEPLOY + " | " + UNDEPLOY
    + " Ressources LoginConfig" + Misc.LINE_SEPARATOR + "Ressources: bar | processId";

  protected Console() {
  }

  public static void deploy(final byte[] bar) throws BonitaException, IOException {
    Misc.checkArgsNotNull(bar);
    ManagementAPI managementAPI = null;
    managementAPI = AccessorUtil.getManagementAPI();
    managementAPI.deployBar(bar);
  }

  public static void undeploy(final String processId) throws BonitaException {
    Misc.checkArgsNotNull(processId);
    ManagementAPI managementAPI = null;
    QueryDefinitionAPI queryDefinitionAPI = null;
    managementAPI = AccessorUtil.getManagementAPI();
    queryDefinitionAPI = AccessorUtil.getQueryDefinitionAPI();

    ProcessDefinition processDefinition = queryDefinitionAPI.getLastProcess(processId);
    if (processDefinition == null) {
      throw new ProcessNotFoundException("can't find package: " + processId);
    }
    PackageDefinitionUUID packageDefinitionUUID = processDefinition.getPackageDefinitionUUID();
    managementAPI.undeploy(packageDefinitionUUID);
  }

  public static void main(String... args) throws LoginException, BonitaException, IOException {
    if (args == null || (args.length != 3 && args.length != 5)) {
      System.err.println(USAGE);
      throw new IllegalArgumentException(USAGE);
    }
    final Misc.NullCheckResult result = Misc.findNull((Object[]) args);

    if (result.hasNull()) {
      System.err.println(USAGE);
      throw new IllegalArgumentException("Some parameters are null: " + Misc.componentsToString(args, false));
    }
    String userName = null;
    String password = null;
    LoginContext loginContext = null;
    //final String environnement = args[0];
    final String action = args[0];
    final String loginConfig = args[2];
    if (args.length == 5) {
      userName = args[3];
      password = args[4];
    }
    if ((userName != null) && (password != null)) {
      loginContext = new LoginContext(loginConfig, new SimpleCallbackHandler(userName, password));
    } else {
      loginContext = new LoginContext(loginConfig, new StandardCallbackHandler());
    }
    loginContext.login();
    if (action.equals(DEPLOY)) {
      final String barFile = args[1];
      final byte[] bar = Misc.getAllContentFrom(new File(barFile));
      deploy(bar);
    } else if (action.equals(UNDEPLOY)) {
      final String processId = args[1];
      undeploy(processId);
    } else {
      System.err.println(ACTION_USAGE);
      throw new IllegalArgumentException(ACTION_USAGE);
    }
    loginContext.logout();
  }
}
