package org.ow2.bonita.util;

import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URL;
import java.util.jar.JarEntry;
import java.util.jar.JarOutputStream;

import org.ow2.novabpm.util.Misc;

public final class BarUtil {

  private BarUtil() {

  }

  public static byte[] generateBarContent(final String xpdlFileName,
      final byte[] xpdlContent, final Class< ? >... classes) throws IOException {
    final ByteArrayOutputStream baos = new ByteArrayOutputStream();
    final JarOutputStream out = new JarOutputStream(new BufferedOutputStream(baos));
    JarEntry entry = null;
    if (xpdlFileName != null && xpdlContent != null) {
      //
      // Add XPDL file to bar file
      //
      entry = new JarEntry(xpdlFileName);
      out.putNextEntry(entry);
      out.write(xpdlContent);
    }
    //
    // add classDatas to bar file
    //
    if (classes != null) {
      for (Class< ? > clazz : classes) {
        entry = new JarEntry(clazz.getName().replace('.', File.separatorChar) + ".class");
        out.putNextEntry(entry);
        out.write(Tool.getClassData(clazz));
      }
    }
    out.flush();
    Misc.close(out);
    return baos.toByteArray();
  }

  public static void generateBar(final String barFile, final String xpdlFileName, final byte[] xpdlContent,
      final Class< ? >... classes) throws IOException {
    byte[] barContent = generateBarContent(xpdlFileName, xpdlContent, classes);
    generateBarFile(barFile, barContent);
  }

  public static byte[] generateBarContent(final String xpdlFileName,
      final Class< ? >... classes) throws BonitaException, IOException {
    final URL url = Thread.currentThread().getContextClassLoader().getResource(xpdlFileName);
    final byte[] xpdlContent = Misc.getAllContentFrom(url);
    return BarUtil.generateBarContent(xpdlFileName, xpdlContent, classes);
  }

  public static byte[] generateBarForClasses(final Class< ? >... classes) throws BonitaException, IOException {
    return generateBarContent(null, null, classes);
  }

  public static void generateBarFile(final String barFile,
      final String xpdlFileName, final Class< ? >... classes) throws BonitaException, IOException {
    byte[] barContent = generateBarContent(xpdlFileName, classes);
    generateBarFile(barFile, barContent);
  }

  private static void generateBarFile(final String barFile, byte[] barContent) throws IOException {
    FileOutputStream archive = new FileOutputStream(barFile);
    archive.write(barContent);
    archive.flush();
    Misc.close(archive);
  }

}
