/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.util;

import org.hibernate.cfg.Configuration;
import org.hibernate.tool.hbm2ddl.SchemaExport;
import org.jbpm.pvm.env.EnvironmentFactory;
import org.ow2.bonita.env.GlobalEnvironmentFactory;


/**
 * @author Guillaume Porcher
 * Helper class to initialize DB tables from the environment.
 */
public final class DbTool {

  private DbTool() { }

  private static SchemaExport getSchemaExport() {
    EnvironmentFactory envFactory = GlobalEnvironmentFactory.getEnvironmentFactory();
    if (envFactory == null) {
      throw new BonitaRuntimeException("Unable to find EnvironmentFactory.");
    }
    Configuration config = envFactory.get(Configuration.class);
    if (config == null) {
      throw new BonitaRuntimeException("Unable to find a hibernate configuration in the EnvironmentFactory.");
    }
    return new SchemaExport(config, config.buildSettings());
  }
  
  /**
   * Export DB schema to the database defined in the environment.
   */
  public static void recreateDb() {
    SchemaExport schemaExport = getSchemaExport();
    schemaExport.create(false, true);
  }
  
  public static void dropDb() {
    SchemaExport schemaExport = getSchemaExport();
    schemaExport.drop(false, true);
  }
  
  public static void main(String[] args) {
    recreateDb();
  }
}
