package org.ow2.bonita.util;

import java.lang.reflect.Method;

import org.ow2.bonita.facade.APIAccessor;
import org.ow2.bonita.facade.ejb.ejb2.EJB2ClientAPIAccessorImpl;

/**
 * Util class used by the engine to access EJB2 API Accessor and manage invocation exception
 * @author Thomas Gueze
 *
 */
public final class EJBUtil {

  private EJBUtil() { }

  public static final APIAccessor EJB2_ACCESSOR = new EJB2ClientAPIAccessorImpl();
  
  public static APIAccessor getEJB2APIAccessor() {
    return EJB2_ACCESSOR;
  }
  
  /**
   * Throw the throwable passed if it's an RuntimeException or if it's an exception declared by the method
   * If not, throw nothing and return
   */
  @SuppressWarnings("unchecked")
  public static void manageInvokeExceptionCause(Method m, Throwable t) throws BonitaException {
    if (t instanceof RuntimeException) {
      throw (RuntimeException) t;
    }
    Class[] throwedClasses = m.getExceptionTypes();
    for (Class clazz : throwedClasses) {
      throwEx(clazz, t);
    }
  }
  
  @SuppressWarnings("unchecked")
  private static <T extends BonitaException> void throwEx(Class<T> clazz, Throwable t) throws T {
    if (t != null && clazz.isAssignableFrom(t.getClass())) {
      throw (T) t;
    }
  }
}
